@@ -43,4 +43,11 @@ public final class TreeNodeWrapper {
     public String toString() {
         return label;
     }
-}
\ No newline at end of file
+
+    /**
+     * @return the label
+     */
+    public String getLabel() {
+        return label;
+    }
+}
