@@ -28,6 +28,9 @@ import java.awt.event.ActionListener;
 import java.io.FileNotFoundException;
 import java.io.FileWriter;
 import java.io.IOException;
+import java.text.DecimalFormat;
+import java.text.Format;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
@@ -65,6 +68,7 @@ import org.apache.jmeter.gui.action.ActionRouter;
 import org.apache.jmeter.gui.action.SaveGraphics;
 import org.apache.jmeter.gui.util.FileDialoger;
 import org.apache.jmeter.gui.util.FilePanel;
+import org.apache.jmeter.gui.util.HeaderAsPropertyRenderer;
 import org.apache.jmeter.gui.util.VerticalPanel;
 import org.apache.jmeter.samplers.Clearable;
 import org.apache.jmeter.samplers.SampleResult;
@@ -92,24 +96,50 @@ import org.apache.log.Logger;
 public class StatGraphVisualizer extends AbstractVisualizer implements Clearable, ActionListener {
     private static final long serialVersionUID = 240L;
 
+    private static final String pct1Label = JMeterUtils.getPropDefault("aggregate_rpt_pct1", "90");
+    private static final String pct2Label = JMeterUtils.getPropDefault("aggregate_rpt_pct2", "95");
+    private static final String pct3Label = JMeterUtils.getPropDefault("aggregate_rpt_pct3", "99");
+    
+    private static final Float pct1Value = new Float(Float.parseFloat(pct1Label)/100);
+    private static final Float pct2Value =  new Float(Float.parseFloat(pct2Label)/100);
+    private static final Float pct3Value =  new Float(Float.parseFloat(pct3Label)/100);
+
     private static final Logger log = LoggingManager.getLoggerForClass();
 
-    private final String[] COLUMNS = { JMeterUtils.getResString("sampler_label"), //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_count"),         //$NON-NLS-1$
-            JMeterUtils.getResString("average"),                        //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_median"),        //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_90%_line"),      //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_min"),           //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_max"),           //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_error%"),        //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_rate"),          //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_bandwidth") };   //$NON-NLS-1$
+    static final String[] COLUMNS = { 
+            "sampler_label",                  //$NON-NLS-1$
+            "aggregate_report_count",         //$NON-NLS-1$
+            "average",                        //$NON-NLS-1$
+            "aggregate_report_median",        //$NON-NLS-1$
+            "aggregate_report_xx_pct1_line",      //$NON-NLS-1$
+            "aggregate_report_xx_pct2_line",      //$NON-NLS-1$
+            "aggregate_report_xx_pct3_line",      //$NON-NLS-1$
+            "aggregate_report_min",           //$NON-NLS-1$
+            "aggregate_report_max",           //$NON-NLS-1$
+            "aggregate_report_error%",        //$NON-NLS-1$
+            "aggregate_report_rate",          //$NON-NLS-1$
+            "aggregate_report_bandwidth" };   //$NON-NLS-1$
+    
+    static final Object[][] COLUMNS_MSG_PARAMETERS = { null, //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            new Object[]{pct1Label},                      //$NON-NLS-1$
+            new Object[]{pct2Label},                      //$NON-NLS-1$
+            new Object[]{pct3Label},                      //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            null,                             //$NON-NLS-1$
+            null };                           //$NON-NLS-1$
 
-    private final String[] GRAPH_COLUMNS = {JMeterUtils.getResString("average"),//$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_median"),        //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_90%_line"),      //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_min"),           //$NON-NLS-1$
-            JMeterUtils.getResString("aggregate_report_max")};          //$NON-NLS-1$
+    private final String[] GRAPH_COLUMNS = {"average",//$NON-NLS-1$
+            "aggregate_report_median",        //$NON-NLS-1$
+            "aggregate_report_xx_pct1_line",      //$NON-NLS-1$
+            "aggregate_report_xx_pct2_line",      //$NON-NLS-1$
+            "aggregate_report_xx_pct3_line",      //$NON-NLS-1$
+            "aggregate_report_min",           //$NON-NLS-1$
+            "aggregate_report_max"};          //$NON-NLS-1$
 
     private final String TOTAL_ROW_LABEL =
         JMeterUtils.getResString("aggregate_report_total_label");       //$NON-NLS-1$
@@ -239,7 +269,24 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
 
     public StatGraphVisualizer() {
         super();
-        model = new ObjectTableModel(COLUMNS,
+        model = createObjectTableModel();
+        eltList.add(new BarGraph(JMeterUtils.getResString("average"), true, new Color(202, 0, 0)));
+        eltList.add(new BarGraph(JMeterUtils.getResString("aggregate_report_median"), false, new Color(49, 49, 181)));
+        eltList.add(new BarGraph(MessageFormat.format(JMeterUtils.getResString("aggregate_report_xx_pct1_line"),new Object[]{pct1Label}), false, new Color(42, 121, 42)));
+        eltList.add(new BarGraph(MessageFormat.format(JMeterUtils.getResString("aggregate_report_xx_pct2_line"),new Object[]{pct2Label}), false, new Color(242, 226, 8)));
+        eltList.add(new BarGraph(MessageFormat.format(JMeterUtils.getResString("aggregate_report_xx_pct3_line"),new Object[]{pct3Label}), false, new Color(202, 10 , 232)));
+        eltList.add(new BarGraph(JMeterUtils.getResString("aggregate_report_min"), false, Color.LIGHT_GRAY));
+        eltList.add(new BarGraph(JMeterUtils.getResString("aggregate_report_max"), false, Color.DARK_GRAY));
+        clearData();
+        init();
+    }
+
+    /**
+     * Creates that Table model 
+     * @return ObjectTableModel
+     */
+    static ObjectTableModel createObjectTableModel() {
+        return new ObjectTableModel(COLUMNS,
                 SamplingStatCalculator.class,
                 new Functor[] {
                 new Functor("getLabel"),                    //$NON-NLS-1$
@@ -247,37 +294,54 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
                 new Functor("getMeanAsNumber"),                //$NON-NLS-1$
                 new Functor("getMedian"),                    //$NON-NLS-1$
                 new Functor("getPercentPoint",                //$NON-NLS-1$
-                new Object[] { new Float(.900) }),
+                        new Object[] { pct1Value }),
+                new Functor("getPercentPoint",                //$NON-NLS-1$
+                        new Object[] { pct2Value }),
+                new Functor("getPercentPoint",                //$NON-NLS-1$
+                        new Object[] { pct3Value }),
                 new Functor("getMin"),                        //$NON-NLS-1$
                 new Functor("getMax"),                         //$NON-NLS-1$
                 new Functor("getErrorPercentage"),            //$NON-NLS-1$
                 new Functor("getRate"),                        //$NON-NLS-1$
                 new Functor("getKBPerSecond") },            //$NON-NLS-1$
-                new Functor[] { null, null, null, null, null, null, null, null,    null, null },
-                new Class[] { String.class, Long.class, Long.class, Long.class, Long.class, Long.class,
-                Long.class, String.class, String.class, String.class });
-        eltList.add(new BarGraph("average", true, new Color(202, 0, 0)));
-        eltList.add(new BarGraph("aggregate_report_median", false, new Color(49, 49, 181)));
-        eltList.add(new BarGraph("aggregate_report_90%_line", false, new Color(42, 121, 42)));
-        eltList.add(new BarGraph("aggregate_report_min", false, Color.LIGHT_GRAY));
-        eltList.add(new BarGraph("aggregate_report_max", false, Color.DARK_GRAY));
-        clearData();
-        init();
+                new Functor[] { null, null, null, null, null, null, null, null, null, null, null, null },
+                new Class[] { String.class, Long.class, Long.class, Long.class, Long.class, 
+                            Long.class, Long.class, Long.class, Long.class, String.class, 
+                            String.class, String.class });
     }
 
+    // Column formats
+    static final Format[] FORMATS =
+        new Format[]{
+            null, // Label
+            null, // count
+            null, // Mean
+            null, // median
+            null, // 90%
+            null, // 95%
+            null, // 99%
+            null, // Min
+            null, // Max
+            new DecimalFormat("#0.00%"), // Error %age //$NON-NLS-1$
+            new DecimalFormat("#.0"),      // Throughput //$NON-NLS-1$
+            new DecimalFormat("#.0")    // pageSize   //$NON-NLS-1$
+        };
+    
     // Column renderers
-    private static final TableCellRenderer[] RENDERERS =
+    static final TableCellRenderer[] RENDERERS =
         new TableCellRenderer[]{
             null, // Label
             null, // count
             null, // Mean
             null, // median
             null, // 90%
+            null, // 95%
+            null, // 99%
             null, // Min
             null, // Max
-            new NumberRenderer("#0.00%"), // Error %age
-            new RateRenderer("#.0"),      // Throughpur
-            new NumberRenderer("#.0"),    // pageSize
+            new NumberRenderer("#0.00%"), // Error %age //$NON-NLS-1$
+            new RateRenderer("#.0"),      // Throughput //$NON-NLS-1$
+            new NumberRenderer("#.0"),    // pageSize   //$NON-NLS-1$
         };
 
     public static boolean testFunctors(){
@@ -347,7 +411,9 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
         mainPanel.add(makeTitlePanel());
 
         myJTable = new JTable(model);
-        myJTable.setPreferredScrollableViewportSize(new Dimension(500, 80));
+        // Fix centering of titles
+        myJTable.getTableHeader().setDefaultRenderer(new HeaderAsPropertyRenderer(COLUMNS_MSG_PARAMETERS));
+        myJTable.setPreferredScrollableViewportSize(new Dimension(500, 70));
         RendererUtils.applyRenderers(myJTable, RENDERERS);
         myScrollPane = new JScrollPane(myJTable);
 
@@ -424,7 +490,7 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
         graphPanel.setMaxLength(maxLength);
         graphPanel.setMaxYAxisScale(maxYAxisScale);
         graphPanel.setXAxisLabels(getAxisLabels());
-        graphPanel.setXAxisTitle((String) columnsList.getSelectedItem());
+        graphPanel.setXAxisTitle(JMeterUtils.getResString((String) columnsList.getSelectedItem()));
         graphPanel.setYAxisLabels(this.yAxisLabel);
         graphPanel.setYAxisTitle(this.yAxisTitle);
         graphPanel.setLegendLabels(getLegendLabels());
@@ -526,9 +592,11 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
      * We use this method to get the data, since we are using
      * ObjectTableModel, so the calling getDataVector doesn't
      * work as expected.
+     * @param model {@link ObjectTableModel}
+     * @param formats Array of {@link Format} array can contain null formatters in this case value is added as is
      * @return the data from the model
      */
-    public List<List<Object>> getAllTableData() {
+    public static List<List<Object>> getAllTableData(ObjectTableModel model, Format[] formats) {
         List<List<Object>> data = new ArrayList<List<Object>>();
         if (model.getRowCount() > 0) {
             for (int rw=0; rw < model.getRowCount(); rw++) {
@@ -537,7 +605,11 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
                 data.add(column);
                 for (int idx=0; idx < cols; idx++) {
                     Object val = model.getValueAt(rw,idx);
-                    column.add(val);
+                    if(formats[idx] != null) {
+                        column.add(formats[idx].format(val));
+                    } else {
+                        column.add(val);
+                    }
                 }
             }
         }
@@ -567,11 +639,11 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
             FileWriter writer = null;
             try {
                 writer = new FileWriter(chooser.getSelectedFile()); // TODO Charset ?
-                CSVSaveService.saveCSVStats(getAllTableData(),writer,saveHeaders.isSelected() ? COLUMNS : null);
+                CSVSaveService.saveCSVStats(getAllTableData(model, FORMATS),writer,saveHeaders.isSelected() ? getLabels(COLUMNS) : null);
             } catch (FileNotFoundException e) {
-                log.warn(e.getMessage());
+                JMeterUtils.reportErrorToUser(e.getMessage(), "Error saving data");
             } catch (IOException e) {
-                log.warn(e.getMessage());
+                JMeterUtils.reportErrorToUser(e.getMessage(), "Error saving data");
             } finally {
                 JOrphanUtils.closeQuietly(writer);
             }
@@ -639,6 +711,19 @@ public class StatGraphVisualizer extends AbstractVisualizer implements Clearable
         }
     }
 
+    /**
+     * 
+     * @param keys I18N keys
+     * @return labels
+     */
+    static String[] getLabels(String[] keys) {
+        String[] labels = new String[keys.length];
+        for (int i = 0; i < labels.length; i++) {
+            labels[i]=MessageFormat.format(JMeterUtils.getResString(keys[i]), COLUMNS_MSG_PARAMETERS[i]);
+        }
+        return labels;
+    }
+
     private void actionMakeGraph() {
         if (model.getRowCount() > 1) {
             makeGraph();
