@@ -36,14 +36,19 @@ import javax.swing.SwingUtilities;
 
 import org.apache.jmeter.engine.util.ValueReplacer;
 import org.apache.jmeter.exceptions.IllegalUserActionException;
+import org.apache.jmeter.gui.UndoHistory.HistoryListener;
 import org.apache.jmeter.gui.tree.JMeterTreeListener;
 import org.apache.jmeter.gui.tree.JMeterTreeModel;
 import org.apache.jmeter.gui.tree.JMeterTreeNode;
+import org.apache.jmeter.gui.util.JMeterToolBar;
 import org.apache.jmeter.services.FileServer;
 import org.apache.jmeter.testbeans.TestBean;
 import org.apache.jmeter.testbeans.gui.TestBeanGUI;
 import org.apache.jmeter.testelement.TestElement;
 import org.apache.jmeter.testelement.TestPlan;
+import org.apache.jmeter.testelement.property.JMeterProperty;
+import org.apache.jmeter.testelement.property.PropertyIterator;
+import org.apache.jmeter.testelement.property.TestElementProperty;
 import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jmeter.util.LocaleChangeEvent;
 import org.apache.jmeter.util.LocaleChangeListener;
@@ -60,7 +65,7 @@ import org.apache.log.Logger;
  * parts of the GUI.
  *
  */
-public final class GuiPackage implements LocaleChangeListener {
+public final class GuiPackage implements LocaleChangeListener, HistoryListener {
     /** Logging. */
     private static final Logger log = LoggingManager.getLoggerForClass();
 
@@ -104,7 +109,7 @@ public final class GuiPackage implements LocaleChangeListener {
 
     /** The main JMeter frame. */
     private MainFrame mainFrame;
-    
+
     /** The main JMeter toolbar. */
     private JToolBar toolbar;
 
@@ -121,15 +126,21 @@ public final class GuiPackage implements LocaleChangeListener {
      */
     private LoggerPanel loggerPanel;
 
-    
+    /**
+     * History for tree states
+     */
+    private UndoHistory undoHistory = new UndoHistory();
+
     /**
      * Private constructor to permit instantiation only from within this class.
      * Use {@link #getInstance()} to retrieve a singleton instance.
      */
     private GuiPackage(JMeterTreeModel treeModel, JMeterTreeListener treeListener) {
         this.treeModel = treeModel;
+        if(undoHistory.isEnabled()) {
+            this.treeModel.addTreeModelListener(undoHistory);
+        }
         this.treeListener = treeListener;
-        JMeterUtils.addLocaleChangeListener(this);
     }
 
     /**
@@ -140,6 +151,18 @@ public final class GuiPackage implements LocaleChangeListener {
     public static GuiPackage getInstance() {
         return guiPack;
     }
+    
+    /**
+     * Register as listener of:
+     * - UndoHistory
+     * - Locale Changes
+     */
+    public void registerAsListener() {
+        if(undoHistory.isEnabled()) {
+            this.undoHistory.registerHistoryListener(this);
+        }
+        JMeterUtils.addLocaleChangeListener(this);
+    }
 
     /**
      * When GuiPackage is requested for the first time, it should be given
@@ -155,6 +178,7 @@ public final class GuiPackage implements LocaleChangeListener {
     public static GuiPackage getInstance(JMeterTreeListener listener, JMeterTreeModel treeModel) {
         if (guiPack == null) {
             guiPack = new GuiPackage(treeModel, listener);
+            guiPack.undoHistory.add(treeModel, "Created");
         }
         return guiPack;
     }
@@ -204,10 +228,10 @@ public final class GuiPackage implements LocaleChangeListener {
      * @param node
      *            the test element which this GUI is being created for
      * @param guiClass
-     *            the fully qualifed class name of the GUI component which will
+     *            the fully qualified class name of the GUI component which will
      *            be created if it doesn't already exist
      * @param testClass
-     *            the fully qualifed class name of the test elements which have
+     *            the fully qualified class name of the test elements which have
      *            to be edited by the returned GUI component
      *
      * @return the GUI component corresponding to the specified test element
@@ -408,8 +432,12 @@ public final class GuiPackage implements LocaleChangeListener {
                 log.debug("Updating current node " + currentNode.getName());
                 JMeterGUIComponent comp = getGui(currentNode.getTestElement());
                 TestElement el = currentNode.getTestElement();
+                int before = getTestElementCheckSum(el);
                 comp.modifyTestElement(el);
-                currentNode.nameChanged(); // Bug 50221 - ensure label is updated
+                int after = getTestElementCheckSum(el);
+                if (before != after) {
+                    currentNode.nameChanged(); // Bug 50221 - ensure label is updated
+                }
             }
             // The current node is now updated
             currentNodeUpdated = true;
@@ -464,7 +492,10 @@ public final class GuiPackage implements LocaleChangeListener {
      *             if a subtree cannot be added to the currently selected node
      */
     public HashTree addSubTree(HashTree subTree) throws IllegalUserActionException {
-        return treeModel.addSubTree(subTree, treeListener.getCurrentNode());
+        HashTree hashTree = treeModel.addSubTree(subTree, treeListener.getCurrentNode());
+        undoHistory.clear();
+        undoHistory.add(this.treeModel, "Loaded tree");
+        return hashTree;
     }
 
     /**
@@ -527,7 +558,7 @@ public final class GuiPackage implements LocaleChangeListener {
     public JMeterTreeListener getTreeListener() {
         return treeListener;
     }
-    
+
     /**
      * Set the main JMeter toolbar.
      *
@@ -546,7 +577,7 @@ public final class GuiPackage implements LocaleChangeListener {
     public JToolBar getMainToolbar() {
         return toolbar;
     }
-    
+
     /**
      * Set the menu item toolbar.
      *
@@ -643,6 +674,7 @@ public final class GuiPackage implements LocaleChangeListener {
      * title and used on saving.
      *
      * @param f
+     *            The filepath of the current test plan
      */
     public void setTestPlanFile(String f) {
         testPlanFile = f;
@@ -670,6 +702,8 @@ public final class GuiPackage implements LocaleChangeListener {
         getTreeModel().clearTestPlan();
         nodesToGui.clear();
         setTestPlanFile(null);
+        undoHistory.clear();
+        undoHistory.add(this.treeModel, "Initial Tree");
     }
 
     /**
@@ -680,6 +714,8 @@ public final class GuiPackage implements LocaleChangeListener {
     public void clearTestPlan(TestElement element) {
         getTreeModel().clearTestPlan(element);
         removeNode(element);
+        undoHistory.clear();
+        undoHistory.add(this.treeModel, "Initial Tree");
     }
 
     public static void showErrorMessage(final String message, final String title){
@@ -720,18 +756,20 @@ public final class GuiPackage implements LocaleChangeListener {
             }
         }
     }
-    
+
     /**
      * Register process to stop on reload
+     * 
      * @param stoppable
+     *            The {@link Stoppable} to be registered
      */
     public void register(Stoppable stoppable) {
         stoppables.add(stoppable);
     }
 
     /**
-     * 
-     * @return List<IStoppable> Copy of IStoppable
+     *
+     * @return copy of list of {@link Stoppable}s
      */
     public List<Stoppable> getStoppables() {
         ArrayList<Stoppable> list = new ArrayList<Stoppable>();
@@ -741,12 +779,12 @@ public final class GuiPackage implements LocaleChangeListener {
 
     /**
      * Set the menu item LoggerPanel.
-     * @param menuItemLoggerPanel
+     * @param menuItemLoggerPanel The menu item LoggerPanel
      */
     public void setMenuItemLoggerPanel(JCheckBoxMenuItem menuItemLoggerPanel) {
         this.menuItemLoggerPanel = menuItemLoggerPanel;
     }
-    
+
     /**
      * Get the menu item LoggerPanel.
      *
@@ -769,4 +807,61 @@ public final class GuiPackage implements LocaleChangeListener {
     public LoggerPanel getLoggerPanel() {
         return loggerPanel;
     }
-}
\ No newline at end of file
+
+    /**
+     * Navigate back and forward through undo history
+     *
+     * @param offset int
+     */
+    public void goInHistory(int offset) {
+        undoHistory.moveInHistory(offset, this.treeModel);
+    }
+
+    /**
+     * @return true if history contains redo item
+     */
+    public boolean canRedo() {
+        return undoHistory.canRedo();
+    }
+
+    /**
+     * @return true if history contains undo item
+     */
+    public boolean canUndo() {
+        return undoHistory.canUndo();
+    }
+
+    /**
+     * Compute checksum of TestElement to detect changes
+     * the method calculates properties checksum to detect testelement
+     * modifications
+     * TODO would be better to override hashCode for TestElement, but I decided not to touch it
+     *
+     * @param el {@link TestElement}
+     * @return int checksum
+     */
+    private int getTestElementCheckSum(TestElement el) {
+        int ret = el.getClass().hashCode();
+        PropertyIterator it = el.propertyIterator();
+        while (it.hasNext()) {
+            JMeterProperty obj = it.next();
+            if (obj instanceof TestElementProperty) {
+                ret ^= getTestElementCheckSum(((TestElementProperty) obj)
+                        .getElement());
+            } else {
+                ret ^= obj.getName().hashCode();
+                ret ^= obj.getStringValue().hashCode();
+            }
+        }
+        return ret;
+    }
+
+    /**
+     * Called when history changes, it updates toolbar
+     */
+    @Override
+    public void notifyChangeInHistory(UndoHistory history) {
+        ((JMeterToolBar)toolbar).updateUndoRedoIcons(history.canUndo(), history.canRedo());
+    }
+
+}
