@@ -39,7 +39,6 @@ import javax.swing.JMenuItem;
 import javax.swing.JPopupMenu;
 import javax.swing.KeyStroke;
 import javax.swing.MenuElement;
-
 import org.apache.jmeter.control.Controller;
 import org.apache.jmeter.gui.GuiPackage;
 import org.apache.jmeter.gui.JMeterGUIComponent;
@@ -206,6 +205,10 @@ public final class MenuFactory {
      * @param addSaveTestFragmentMenu Add Save as Test Fragment menu if true 
      */
     public static void addFileMenu(JPopupMenu menu, boolean addSaveTestFragmentMenu) {
+        // the undo/redo as a standard goes first in Edit menus
+        // maybe there's better place for them in JMeter?
+        addUndoItems(menu);
+
         addSeparator(menu);
         menu.add(makeMenuItemRes("open", ActionNames.OPEN));// $NON-NLS-1$
         menu.add(makeMenuItemRes("menu_merge", ActionNames.MERGE));// $NON-NLS-1$
@@ -247,6 +250,24 @@ public final class MenuFactory {
         menu.add(makeMenuItemRes("help", ActionNames.HELP));// $NON-NLS-1$
     }
 
+    /**
+     * Add undo / redo
+     * @param menu JPopupMenu
+     */
+    private static void addUndoItems(JPopupMenu menu) {
+        addSeparator(menu);
+
+        JMenuItem undo = makeMenuItemRes("undo", ActionNames.UNDO); //$NON-NLS-1$
+        undo.setEnabled(GuiPackage.getInstance().canUndo());
+        menu.add(undo);
+
+        JMenuItem redo = makeMenuItemRes("redo", ActionNames.REDO); //$NON-NLS-1$
+        // TODO: we could even show some hints on action being undone here if this will be required (by passing those hints into history  records)
+        redo.setEnabled(GuiPackage.getInstance().canRedo());
+        menu.add(redo);
+    }
+
+
     public static JMenu makeMenus(String[] categories, String label, String actionCommand) {
         JMenu addMenu = new JMenu(label);
         for (int i = 0; i < categories.length; i++) {
@@ -345,7 +366,7 @@ public final class MenuFactory {
      * @param menuInfo - collection of MenuInfo items
      * @param actionCommand - predefined string, e.g. ActionNames.ADD
      *     @see org.apache.jmeter.gui.action.ActionNames
-     * @param menuName
+     * @param menuName The name of the newly created menu
      * @return the menu
      */
     public static JMenu makeMenu(Collection<MenuInfo> menuInfo, String actionCommand, String menuName) {
@@ -554,8 +575,10 @@ public final class MenuFactory {
      * Used by Merge
      *
      * @param parentNode
-     * @param element - top-level test element to be added
-     *
+     *            The {@link JMeterTreeNode} to test, if a new element can be
+     *            added to it
+     * @param element
+     *            - top-level test element to be added
      * @return whether it is OK to add the element to this parent
      */
     public static boolean canAddTo(JMeterTreeNode parentNode, TestElement element) {
@@ -569,8 +592,10 @@ public final class MenuFactory {
      * Used by DragNDrop and Paste.
      *
      * @param parentNode
-     * @param nodes - array of nodes that are to be added
-     *
+     *            The {@link JMeterTreeNode} to test, if <code>nodes[]</code>
+     *            can be added to it
+     * @param nodes
+     *            - array of nodes that are to be added
      * @return whether it is OK to add the dragged nodes to this parent
      */
     public static boolean canAddTo(JMeterTreeNode parentNode, JMeterTreeNode nodes[]) {
@@ -587,8 +612,9 @@ public final class MenuFactory {
 
         // Force TestFragment to only be pastable under a Test Plan
         if (foundClass(nodes, new Class[]{org.apache.jmeter.control.TestFragmentController.class})){
-            if (parent instanceof TestPlan)
+            if (parent instanceof TestPlan) {
                 return true;
+            }
             return false;
         }
 
@@ -699,9 +725,9 @@ public final class MenuFactory {
             String lab1 = o1.getLabel();
             String lab2 = o2.getLabel();
             if (caseBlind) {
-                return lab1.toLowerCase(Locale.ENGLISH).compareTo(lab2.toLowerCase(Locale.ENGLISH));                
+                return lab1.toLowerCase(Locale.ENGLISH).compareTo(lab2.toLowerCase(Locale.ENGLISH));
             }
-            return lab1.compareTo(lab2);                
+            return lab1.compareTo(lab2);
         }
     }
 
