@@ -166,6 +166,10 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         this(null);
     }
 
+    /**
+     * Constructor which sets the used {@link Summariser}
+     * @param summer The {@link Summariser} to use
+     */
     public ResultCollector(Summariser summer) {
         setErrorLogging(false);
         setSuccessOnlyLogging(false);
@@ -188,18 +192,40 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         setProperty(FILENAME, f);
     }
 
+    /**
+     * Get the filename of the file this collector uses
+     * 
+     * @return The name of the file
+     */
     public String getFilename() {
         return getPropertyAsString(FILENAME);
     }
 
+    /**
+     * Get the state of error logging
+     * 
+     * @return Flag whether errors should be logged
+     */
     public boolean isErrorLogging() {
         return getPropertyAsBoolean(ERROR_LOGGING);
     }
 
+    /**
+     * Sets error logging flag
+     * 
+     * @param errorLogging
+     *            The flag whether errors should be logged
+     */
     public final void setErrorLogging(boolean errorLogging) {
         setProperty(new BooleanProperty(ERROR_LOGGING, errorLogging));
     }
 
+    /**
+     * Sets the flag whether only successful samples should be logged
+     * 
+     * @param value
+     *            The flag whether only successful samples should be logged
+     */
     public final void setSuccessOnlyLogging(boolean value) {
         if (value) {
             setProperty(new BooleanProperty(SUCCESS_ONLY_LOGGING, true));
@@ -208,15 +234,22 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         }
     }
 
+    /**
+     * Get the state of successful only logging
+     * 
+     * @return Flag whether only successful samples should be logged
+     */
     public boolean isSuccessOnlyLogging() {
         return getPropertyAsBoolean(SUCCESS_ONLY_LOGGING,false);
     }
 
     /**
-     * Decides whether or not to a sample is wanted based on:<br/>
-     * - errorOnly<br/>
-     * - successOnly<br/>
-     * - sample success<br/>
+     * Decides whether or not to a sample is wanted based on:
+     * <ul>
+     * <li>errorOnly</li>
+     * <li>successOnly</li>
+     * <li>sample success</li>
+     * </ul>
      * Should only be called for single samples.
      *
      * @param success is sample successful
@@ -229,10 +262,12 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     }
 
     /**
-     * Decides whether or not to a sample is wanted based on: <br/>
-     * - errorOnly <br/>
-     * - successOnly <br/>
-     * - sample success <br/>
+     * Decides whether or not to a sample is wanted based on:
+     * <ul>
+     * <li>errorOnly</li>
+     * <li>successOnly</li>
+     * <li>sample success</li>
+     * </ul>
      * This version is intended to be called by code that loops over many samples;
      * it is cheaper than fetching the settings each time.
      * @param success status of sample
@@ -266,7 +301,12 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
             instanceCount--;
             if (instanceCount <= 0) {
                 // No need for the hook now
-                Runtime.getRuntime().removeShutdownHook(shutdownHook);
+                // Bug 57088 - prevent (im?)possible NPE
+                if (shutdownHook != null) {
+                    Runtime.getRuntime().removeShutdownHook(shutdownHook);
+                } else {
+                    log.warn("Should not happen: shutdownHook==null, instanceCount=" + instanceCount);
+                }
                 finalizeFileOutput();
                 inTest = false;
             }
@@ -314,9 +354,11 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     /**
      * Loads an existing sample data (JTL) file.
      * This can be one of:
-     * - XStream format
-     * - Avalon format
-     * - CSV format
+     * <ul>
+     *   <li>XStream format</li>
+     *   <li>Avalon format</li>
+     *   <li>CSV format</li>
+     * </ul>
      *
      */
     public void loadExistingFile() {
@@ -557,11 +599,11 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     /**
      * recordStats is used to save statistics generated by visualizers
      *
-     * @param e
-     * @throws Exception
+     * @param e The data to save
+     * @throws IOException when data writing fails
      */
     // Used by: MonitorHealthVisualizer.add(SampleResult res)
-    public void recordStats(TestElement e) throws Exception {
+    public void recordStats(TestElement e) throws IOException {
         if (out != null) {
             SaveService.saveTestElement(e, out);
         }
@@ -591,6 +633,16 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         }
     }
 
+    /**
+     * Flush PrintWriter to synchronize file contents
+     */
+    protected void flushFile() {
+        if (out != null) {
+            log.info("forced flush through ResultCollecto#flushFile");
+            out.flush();
+        }
+    }
+
     /**
      * Flush PrintWriter, called by Shutdown Hook to ensure no data is lost
      */
@@ -644,4 +696,4 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     @Override
     public void clearData() {
     }
-}
\ No newline at end of file
+}
