@@ -55,12 +55,13 @@ import org.apache.log.Logger;
  * Holds details of which sample attributes to save.
  *
  * The pop-up dialogue for this is created by the class SavePropertyDialog, which assumes:
- * For each field XXX
- * - methods have the signature "boolean saveXXX()"
- * - a corresponding "void setXXX(boolean)" method
- * - messages.properties contains the key save_XXX
- *
- *
+ * <p>
+ * For each field <em>XXX</em>
+ * <ul>
+ *  <li>methods have the signature "boolean save<em>XXX</em>()"</li>
+ *  <li>a corresponding "void set<em>XXX</em>(boolean)" method</li>
+ *  <li>messages.properties contains the key save_<em>XXX</em></li>
+ * </ul>
  */
 public class SampleSaveConfiguration implements Cloneable, Serializable {
     private static final long serialVersionUID = 7L;
@@ -197,6 +198,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
     private static final String SUBRESULTS_PROP      = "jmeter.save.saveservice.subresults"; // $NON_NLS-1$
     private static final String ASSERTIONS_PROP      = "jmeter.save.saveservice.assertions"; // $NON_NLS-1$
     private static final String LATENCY_PROP         = "jmeter.save.saveservice.latency"; // $NON_NLS-1$
+    private static final String CONNECT_TIME_PROP    = "jmeter.save.saveservice.connect_time"; // $NON_NLS-1$
     private static final String SAMPLERDATA_PROP     = "jmeter.save.saveservice.samplerData"; // $NON_NLS-1$
     private static final String RESPONSEHEADERS_PROP = "jmeter.save.saveservice.responseHeaders"; // $NON_NLS-1$
     private static final String REQUESTHEADERS_PROP  = "jmeter.save.saveservice.requestHeaders"; // $NON_NLS-1$
@@ -215,7 +217,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
     // N.B. Remember to update the equals and hashCode methods when adding new variables.
 
     // Initialise values from properties
-    private boolean time = _time, latency = _latency, timestamp = _timestamp, success = _success, label = _label,
+    private boolean time = _time, latency = _latency, connectTime=_connectTime, timestamp = _timestamp, success = _success, label = _label,
             code = _code, message = _message, threadName = _threadName, dataType = _dataType, encoding = _encoding,
             assertions = _assertions, subresults = _subresults, responseData = _responseData,
             samplerData = _samplerData, xml = _xml, fieldNames = _fieldNames, responseHeaders = _responseHeaders,
@@ -249,7 +251,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
 
     // Defaults from properties:
     private static final boolean _time, _timestamp, _success, _label, _code, _message, _threadName, _xml,
-            _responseData, _dataType, _encoding, _assertions, _latency, _subresults, _samplerData, _fieldNames,
+            _responseData, _dataType, _encoding, _assertions, _latency, _connectTime, _subresults, _samplerData, _fieldNames,
             _responseHeaders, _requestHeaders;
 
     private static final boolean _responseDataOnError;
@@ -302,6 +304,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
         _subresults      = TRUE.equalsIgnoreCase(props.getProperty(SUBRESULTS_PROP, TRUE));
         _assertions      = TRUE.equalsIgnoreCase(props.getProperty(ASSERTIONS_PROP, TRUE));
         _latency         = TRUE.equalsIgnoreCase(props.getProperty(LATENCY_PROP, TRUE));
+        _connectTime     = TRUE.equalsIgnoreCase(props.getProperty(CONNECT_TIME_PROP, FALSE));
         _samplerData     = TRUE.equalsIgnoreCase(props.getProperty(SAMPLERDATA_PROP, FALSE));
         _responseHeaders = TRUE.equalsIgnoreCase(props.getProperty(RESPONSEHEADERS_PROP, FALSE));
         _requestHeaders  = TRUE.equalsIgnoreCase(props.getProperty(REQUESTHEADERS_PROP, FALSE));
@@ -393,7 +396,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
             _xml = false;
         }
 
-        _threadCounts=TRUE.equalsIgnoreCase(props.getProperty(SAVE_THREAD_COUNTS, FALSE));
+        _threadCounts=TRUE.equalsIgnoreCase(props.getProperty(SAVE_THREAD_COUNTS, TRUE));
 
         _sampleCount=TRUE.equalsIgnoreCase(props.getProperty(SAVE_SAMPLE_COUNT, FALSE));
 
@@ -440,6 +443,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
         hostname = value;
         label = value;
         latency = value;
+        connectTime = value;
         message = value;
         printMilliseconds = _printMilliseconds;//is derived from properties only
         requestHeaders = value;
@@ -490,6 +494,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
         SampleSaveConfiguration s = (SampleSaveConfiguration)obj;
         boolean primitiveValues = s.time == time &&
             s.latency == latency &&
+            s.connectTime == connectTime &&
             s.timestamp == timestamp &&
             s.success == success &&
             s.label == label &&
@@ -535,6 +540,7 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
         int hash = 7;
         hash = 31 * hash + (time ? 1 : 0);
         hash = 31 * hash + (latency ? 1 : 0);
+        hash = 31 * hash + (connectTime ? 1 : 0);
         hash = 31 * hash + (timestamp ? 1 : 0);
         hash = 31 * hash + (success ? 1 : 0);
         hash = 31 * hash + (label ? 1 : 0);
@@ -634,6 +640,14 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
         this.latency = latency;
     }
 
+    public boolean saveConnectTime() {
+        return connectTime;
+    }
+
+    public void setConnectTime(boolean connectTime) {
+        this.connectTime = connectTime;
+    }
+
     public boolean saveMessage() {
         return message;
     }
@@ -777,6 +791,10 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
 
     /**
      * Only intended for use by OldSaveService (and test cases)
+     * 
+     * @param fmt
+     *            format of the date to be saved. If <code>null</code>
+     *            milliseconds since epoch will be printed
      */
     public void setFormatter(DateFormat fmt){
         printMilliseconds = (fmt == null); // maintain relationship
@@ -834,4 +852,4 @@ public class SampleSaveConfiguration implements Cloneable, Serializable {
     public void setIdleTime(boolean save) {
         idleTime = save;
     }
-}
\ No newline at end of file
+}
