@@ -87,6 +87,7 @@ public final class CSVSaveService {
     private static final String CSV_URL = "URL"; // $NON-NLS-1$
     private static final String CSV_FILENAME = "Filename"; // $NON-NLS-1$
     private static final String CSV_LATENCY = "Latency"; // $NON-NLS-1$
+    private static final String CSV_CONNECT_TIME = "Connect"; // $NON-NLS-1$
     private static final String CSV_ENCODING = "Encoding"; // $NON-NLS-1$
     private static final String CSV_HOSTNAME = "Hostname"; // $NON-NLS-1$
     private static final String CSV_IDLETIME = "IdleTime"; // $NON-NLS-1$
@@ -127,6 +128,8 @@ public final class CSVSaveService {
      * @param resultCollector
      *            the parent collector
      * @throws IOException
+     *             when the file referenced by <code>filename</code> can't be
+     *             read correctly
      */
     public static void processSamples(String filename, Visualizer visualizer,
             ResultCollector resultCollector) throws IOException {
@@ -348,6 +351,11 @@ public final class CSVSaveService {
                 text = parts[i++];
                 result.setIdleTime(Long.parseLong(text));
             }
+            if (saveConfig.saveConnectTime()) {
+                field = CSV_CONNECT_TIME;
+                text = parts[i++];
+                result.setConnectTime(Long.parseLong(text));
+            }
 
             if (i + saveConfig.getVarCount() < parts.length) {
                 log.warn("Line: " + lineNumber + ". Found " + parts.length
@@ -383,6 +391,8 @@ public final class CSVSaveService {
     /**
      * Generates the field names for the output file
      * 
+     * @param saveConfig
+     *            the configuration of what is to be saved
      * @return the field names as a string
      */
     public static String printableFieldNamesToString(
@@ -484,6 +494,11 @@ public final class CSVSaveService {
             text.append(delim);
         }
 
+        if (saveConfig.saveConnectTime()) {
+            text.append(CSV_CONNECT_TIME);
+            text.append(delim);
+        }
+
         for (int i = 0; i < SampleEvent.getVarCount(); i++) {
             text.append(VARIABLE_NAME_QUOTE_CHAR);
             text.append(SampleEvent.getVarName(i));
@@ -537,6 +552,7 @@ public final class CSVSaveService {
         headerLabelMethods.put(CSV_ERROR_COUNT, new Functor("setSampleCount"));
         headerLabelMethods.put(CSV_HOSTNAME, new Functor("setHostname"));
         headerLabelMethods.put(CSV_IDLETIME, new Functor("setIdleTime"));
+        headerLabelMethods.put(CSV_CONNECT_TIME, new Functor("setConnectTime"));
     }
 
     /**
@@ -654,6 +670,7 @@ public final class CSVSaveService {
      * @param writer
      *            output file
      * @throws IOException
+     *             when writing to <code>writer</code> fails
      */
     public static void saveCSVStats(List<?> data, FileWriter writer)
             throws IOException {
@@ -673,6 +690,7 @@ public final class CSVSaveService {
      * @param headers
      *            header names (if non-null)
      * @throws IOException
+     *             when writing to <code>writer</code> fails
      */
     public static void saveCSVStats(List<?> data, FileWriter writer,
             String headers[]) throws IOException {
@@ -710,6 +728,7 @@ public final class CSVSaveService {
      * @param writer
      *            output file
      * @throws IOException
+     *             when writing to <code>writer</code> fails
      */
     public static void saveCSVStats(DefaultTableModel model, FileWriter writer)
             throws IOException {
@@ -718,8 +737,8 @@ public final class CSVSaveService {
 
     /**
      * Method saves aggregate statistics as CSV from a table model. Same as
-     * {@link #saveCSVStats(List, FileWriter, String[])} except that there is
-     * no need to create a List containing the data.
+     * {@link #saveCSVStats(List, FileWriter, String[])} except that there is no
+     * need to create a List containing the data.
      * 
      * @param model
      *            table model containing the data
@@ -728,6 +747,7 @@ public final class CSVSaveService {
      * @param saveHeaders
      *            whether or not to save headers
      * @throws IOException
+     *             when writing to <code>writer</code> fails
      */
     public static void saveCSVStats(DefaultTableModel model, FileWriter writer,
             boolean saveHeaders) throws IOException {
@@ -943,6 +963,10 @@ public final class CSVSaveService {
             text.append(event.getResult().getIdleTime());
         }
 
+        if (saveConfig.saveConnectTime()) {
+            text.append(sample.getConnectTime());
+        }
+
         for (int i = 0; i < SampleEvent.getVarCount(); i++) {
             text.append(event.getVarValue(i));
         }
@@ -1119,7 +1143,7 @@ public final class CSVSaveService {
      * Handles DOS (CRLF), Unix (LF), and Mac (CR) line-endings equally.
      * 
      * @param line
-     *            input line
+     *            input line - not {@code null}
      * @param delim
      *            delimiter (e.g. comma)
      * @return array of strings
