@@ -19,6 +19,7 @@
 package org.apache.jmeter.save;
 
 import java.io.BufferedInputStream;
+import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
@@ -33,8 +34,10 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
+import org.apache.commons.lang3.builder.ToStringBuilder;
 import org.apache.jmeter.reporters.ResultCollectorHelper;
 import org.apache.jmeter.samplers.SampleEvent;
+import org.apache.jmeter.samplers.SampleResult;
 import org.apache.jmeter.testelement.TestElement;
 import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jmeter.util.NameUpdater;
@@ -174,13 +177,13 @@ public class SaveService {
     
     // Must match _version property value in saveservice.properties
     // used to ensure saveservice.properties and SaveService are updated simultaneously
-    private static final String PROPVERSION = "2.6";// Expected version $NON-NLS-1$
+    private static final String PROPVERSION = "2.8";// Expected version $NON-NLS-1$
 
     // Internal information only
     private static String fileVersion = ""; // read from saveservice.properties file// $NON-NLS-1$
     // Must match Revision id value in saveservice.properties, 
     // used to ensure saveservice.properties and SaveService are updated simultaneously
-    private static final String FILEVERSION = "1554411"; // Expected value $NON-NLS-1$
+    private static final String FILEVERSION = "1656252"; // Expected value $NON-NLS-1$
     private static String fileEncoding = ""; // read from properties file// $NON-NLS-1$
 
     static {
@@ -349,6 +352,7 @@ public class SaveService {
      *
      * @param evt sampleResult wrapped in a sampleEvent
      * @param writer output stream which must be created using {@link #getFileEncoding(String)}
+     * @throws IOException when writing data to output fails
      */
     // Used by ResultCollector.sampleOccurred(SampleEvent event)
     public synchronized static void saveSampleResult(SampleEvent evt, Writer writer) throws IOException {
@@ -356,13 +360,31 @@ public class SaveService {
         dh.put(SAMPLE_EVENT_OBJECT, evt);
         // This is effectively the same as saver.toXML(Object, Writer) except we get to provide the DataHolder
         // Don't know why there is no method for this in the XStream class
-        JTLSAVER.marshal(evt.getResult(), new XppDriver().createWriter(writer), dh);
+        try {
+            JTLSAVER.marshal(evt.getResult(), new XppDriver().createWriter(writer), dh);
+        } catch(RuntimeException e) {
+            throw new IllegalArgumentException("Failed marshalling:"+(evt.getResult() != null ? showDebuggingInfo(evt.getResult()) : "null"), e);
+        }
         writer.write('\n');
     }
 
+    /**
+     * 
+     * @param result SampleResult
+     * @return String debugging information
+     */
+    private static String showDebuggingInfo(SampleResult result) {
+        try {
+            return "class:"+result.getClass()+",content:"+ToStringBuilder.reflectionToString(result);
+        } catch(Exception e) {
+            return "Exception occured creating debug from event, message:"+e.getMessage();
+        }
+    }
+
     /**
      * @param elem test element
      * @param writer output stream which must be created using {@link #getFileEncoding(String)}
+     * @throws IOException when writing data to output fails
      */
     // Used by ResultCollector#recordStats()
     public synchronized static void saveTestElement(TestElement elem, Writer writer) throws IOException {
@@ -470,9 +492,9 @@ public class SaveService {
      *
      * @param reader of the file
      * @param resultCollectorHelper helper class to enable TestResultWrapperConverter to deliver the samples
-     * @throws Exception
+     * @throws IOException if an I/O error occurs
      */
-    public static void loadTestResults(InputStream reader, ResultCollectorHelper resultCollectorHelper) throws Exception {
+    public static void loadTestResults(InputStream reader, ResultCollectorHelper resultCollectorHelper) throws IOException {
         // Get the InputReader to use
         InputStreamReader inputStreamReader = getInputStreamReader(reader);
         DataHolder dh = JTLSAVER.newDataHolder();
@@ -485,11 +507,47 @@ public class SaveService {
 
     /**
      * Load a Test tree (JMX file)
-     * @param reader on the JMX file
-     * @return the loaded tree
-     * @throws Exception if there is a problem reading the file or processing it
+     * @param reader the JMX file as an {@link InputStream}
+     * @return the loaded tree or null if an error occurs
+     * @throws IOException if there is a problem reading the file or processing it
+     * @deprecated use {@link SaveService}{@link #loadTree(File)}
      */
-    public static HashTree loadTree(InputStream reader) throws Exception {
+    public static HashTree loadTree(InputStream reader) throws IOException {
+        try {
+            return readTree(reader, null);
+        } catch(IllegalArgumentException e) {
+            log.error("Problem loading XML, message:"+e.getMessage(), e);
+            return null;
+        } finally {
+            JOrphanUtils.closeQuietly(reader);
+        }
+    }
+    
+    /**
+     * Load a Test tree (JMX file)
+     * @param file the JMX file
+     * @return the loaded tree
+     * @throws IOException if there is a problem reading the file or processing it
+     */
+    public static HashTree loadTree(File file) throws IOException {
+        log.info("Loading file: " + file);
+        InputStream reader = null;
+        try {
+            reader = new FileInputStream(file);
+            return readTree(reader, file);
+        } finally {
+            JOrphanUtils.closeQuietly(reader);
+        }
+    }
+
+    /**
+     * 
+     * @param reader {@link InputStream} 
+     * @param file the JMX file used only for debug, can be null
+     * @return the loaded tree
+     * @throws IOException if there is a problem reading the file or processing it
+     */
+    private static final HashTree readTree(InputStream reader, File file) throws IOException {
         if (!reader.markSupported()) {
             reader = new BufferedInputStream(reader);
         }
@@ -506,22 +564,32 @@ public class SaveService {
             }
             return wrapper.testPlan;
         } catch (CannotResolveClassException e) {
+            // FIXME We switching to JAVA7, use Multi-Catch Exceptions
             if (e.getMessage().startsWith("node")) {
                 log.info("Problem loading XML, trying Avalon format");
                 reader.reset();
                 return OldSaveService.loadSubTree(reader);                
             }
-            log.warn("Problem loading XML, cannot determine class for element: " + e.getLocalizedMessage());
-            return null;
+            if(file != null) {
+                throw new IllegalArgumentException("Problem loading XML from:'"+file.getAbsolutePath()+"', cannot determine class for element: " + e, e);
+            } else {
+                throw new IllegalArgumentException("Problem loading XML, cannot determine class for element: " + e, e);
+            }
         } catch (NoClassDefFoundError e) {
-            log.error("Missing class "+e);
-            return null;
+            if(file != null) {
+                throw new IllegalArgumentException("Problem loading XML from:'"+file.getAbsolutePath()+"', missing class "+e , e);
+            } else {
+                throw new IllegalArgumentException("Problem loading XML, missing class "+e , e);
+            }
         } catch (ConversionException e) {
-            log.error("Conversion error "+e);
-            return null;
+            if(file != null) {
+                throw new IllegalArgumentException("Problem loading XML from:'"+file.getAbsolutePath()+"', conversion error "+e , e);
+            } else {
+                throw new IllegalArgumentException("Problem loading XML, conversion error "+e , e);
+            }
         }
-    }
 
+    }
     private static InputStreamReader getInputStreamReader(InputStream inStream) {
         // Check if we have a encoding to use from properties
         Charset charset = getFileEncodingCharset();
@@ -616,4 +684,4 @@ public class SaveService {
     public static String getVERSION() {
         return VERSION;
     }
-}
\ No newline at end of file
+}
