@@ -69,12 +69,12 @@ import org.apache.log.Logger;
  * <dt>notUndefined: Boolean</dt>
  * <dd>If true, the property should not be left undefined. A <b>default</b>
  * attribute must be provided if this is set.</dd>
- * <dd>notExpression: Boolean</dd>
+ * <dt>notExpression: Boolean</dt>
  * <dd>If true, the property content should always be constant: JMeter
- * 'expressions' (strings using ${var}, etc...) can't be used.</dt>
- * <dd>notOther: Boolean</dd>
+ * 'expressions' (strings using ${var}, etc...) can't be used.</dd>
+ * <dt>notOther: Boolean</dt>
  * <dd>If true, the property content must always be one of the tags values or
- * null.</dt>
+ * null.</dd>
  * <dt>default: Object</dt>
  * <dd>Initial value for the property's GUI. Must be provided and be non-null
  * if <b>notUndefined</b> is set. Must be one of the provided tags (or null) if
@@ -99,7 +99,9 @@ public class GenericTestBeanCustomizer extends JPanel implements SharedCustomize
 
     private static final Logger log = LoggingManager.getLoggerForClass();
 
-    // should be quicker to find the editors if they are registered.
+    // Need to register Editors for Java classes because we cannot create them
+    // in the same package, nor can we create them in the built-in search patch of packages,
+    // as that is not part of the public API
     static {
         PropertyEditorManager.registerEditor(Long.class,    LongPropertyEditor.class);
         PropertyEditorManager.registerEditor(Integer.class, IntegerPropertyEditor.class);
@@ -133,6 +135,9 @@ public class GenericTestBeanCustomizer extends JPanel implements SharedCustomize
     /** Whether the field disallows constant values different from the provided tags; Boolean, default FALSE */
     public static final String NOT_OTHER = "notOther"; //$NON-NLS-1$
 
+    /** If specified, create a multi-line editor */
+    public static final String MULTILINE = "multiline";
+
     /** Default value, must be provided if {@link #NOT_UNDEFINED} is TRUE */
     public static final String DEFAULT = "default"; //$NON-NLS-1$
 
@@ -399,7 +404,8 @@ public class GenericTestBeanCustomizer extends JPanel implements SharedCustomize
         if (notNull && tags == null) {
             guiEditor = new FieldStringEditor();
         } else {
-            guiEditor = new ComboStringEditor(tags, notExpression && notOther, notNull);
+            guiEditor = new ComboStringEditor(tags, notExpression && notOther, notNull,
+                    (ResourceBundle) descriptor.getValue(GenericTestBeanCustomizer.RESOURCE_BUNDLE));
         }
 
         WrapperEditor wrapper = new WrapperEditor(typeEditor, guiEditor,
@@ -584,7 +590,8 @@ public class GenericTestBeanCustomizer extends JPanel implements SharedCustomize
             Component customEditor = editors[i].getCustomEditor();
 
             boolean multiLineEditor = false;
-            if (customEditor.getPreferredSize().height > 50 || customEditor instanceof JScrollPane) {
+            if (customEditor.getPreferredSize().height > 50 || customEditor instanceof JScrollPane
+                    || descriptors[i].getValue(MULTILINE) != null) {
                 // TODO: the above works in the current situation, but it's
                 // just a hack. How to get each editor to report whether it
                 // wants to grow bigger? Whether the property label should
