@@ -630,7 +630,7 @@ public abstract class AbstractTestElement implements TestElement, Serializable,
      * {@inheritDoc}}
      */
     @Override
-    public List<String> getSearchableTokens() throws Exception {
+    public List<String> getSearchableTokens() {
         List<String> result = new ArrayList<String>(25);
         PropertyIterator iterator = propertyIterator();
         while(iterator.hasNext()) {
@@ -643,8 +643,8 @@ public abstract class AbstractTestElement implements TestElement, Serializable,
     
     /**
      * Add to result the values of propertyNames
-     * @param result List<String> values of propertyNames
-     * @param propertyNames Set<String> properties to extract
+     * @param result List of values of propertyNames
+     * @param propertyNames Set of names of properties to extract
      */
     protected final void addPropertiesValues(List<String> result, Set<String> propertyNames) {
         PropertyIterator iterator = propertyIterator();
