@@ -27,13 +27,6 @@ public class WorkBench extends AbstractTestElement {
 
     private static final boolean SAVE_WORKBENCH_DEFAULT = false;
 
-    /**
-     * Constructor for the WorkBench object.
-     */
-    public WorkBench(String name, boolean isRootNode) {
-        setName(name);
-    }
-
     public WorkBench() {
     }
 
