@@ -153,6 +153,9 @@ public class JMeterThread implements Runnable, Interruptible {
 
     /**
      * Enable the scheduler for this JMeterThread.
+     *
+     * @param sche
+     *            flag whether the scheduler should be enabled
      */
     public void setScheduled(boolean sche) {
         this.scheduler = sche;
@@ -204,6 +207,7 @@ public class JMeterThread implements Runnable, Interruptible {
         long delay = now - endTime;
         if ((delay >= 0)) {
             running = false;
+            log.info("Stopping because end time detected by thread: " + threadName);
         }
     }
 
@@ -282,6 +286,7 @@ public class JMeterThread implements Runnable, Interruptible {
                 }
                 if (controller.isDone()) {
                     running = false;
+                    log.info("Thread is done: " + threadName);
                 }
             }
         }
@@ -435,6 +440,14 @@ public class JMeterThread implements Runnable, Interruptible {
                     result.setGroupThreads(threadGroup.getNumberOfThreads());
                     result.setAllThreads(JMeterContextService.getNumberOfThreads());
                     result.setThreadName(threadName);
+                    SampleResult[]subResults = result.getSubResults();
+                    if(subResults != null) {
+                        for (SampleResult subResult : subResults) {
+                            subResult.setGroupThreads(threadGroup.getNumberOfThreads());
+                            subResult.setAllThreads(JMeterContextService.getNumberOfThreads());
+                            subResult.setThreadName(threadName);
+                        }
+                    }
                     threadContext.setPreviousResult(result);
                     runPostProcessors(pack.getPostProcessors());
                     checkAssertions(pack.getAssertions(), result, threadContext);
@@ -829,6 +842,8 @@ public class JMeterThread implements Runnable, Interruptible {
 
     /**
      * Returns the threadNum.
+     *
+     * @return the threadNum
      */
     public int getThreadNum() {
         return threadNum;
@@ -857,7 +872,7 @@ public class JMeterThread implements Runnable, Interruptible {
     /**
      * Save the engine instance for access to the stop methods
      *
-     * @param engine
+     * @param engine the engine which is used
      */
     public void setEngine(StandardJMeterEngine engine) {
         this.engine = engine;
