@@ -112,6 +112,7 @@ public class JMeterUtils implements UnitTestManager {
 
     /**
      * Gets Perl5Matcher for this thread.
+     * @return the {@link Perl5Matcher} for this thread
      */
     public static Perl5Matcher getMatcher() {
         return localMatcher.get();
@@ -172,8 +173,10 @@ public class JMeterUtils implements UnitTestManager {
      * Load the JMeter properties file; if not found, then
      * default to "org/apache/jmeter/jmeter.properties" from the classpath
      *
+     * <p>
      * c.f. loadProperties
      *
+     * @param file Name of the file from which the JMeter properties should be loaded
      */
     public static void loadJMeterProperties(String file) {
         Properties p = new Properties(System.getProperties());
@@ -258,7 +261,7 @@ public class JMeterUtils implements UnitTestManager {
     /**
      * Get a compiled expression from the pattern cache (READ_ONLY).
      *
-     * @param expression
+     * @param expression regular expression to be looked up
      * @return compiled pattern
      *
      * @throws MalformedCachePatternException (Runtime)
@@ -273,7 +276,7 @@ public class JMeterUtils implements UnitTestManager {
      * Get a compiled expression from the pattern cache.
      *
      * @param expression RE
-     * @param options e.g. READ_ONLY_MASK
+     * @param options e.g. {@link Perl5Compiler#READ_ONLY_MASK READ_ONLY_MASK}
      * @return compiled pattern
      *
      * @throws MalformedCachePatternException (Runtime)
@@ -298,6 +301,7 @@ public class JMeterUtils implements UnitTestManager {
      *
      * @param superClass - single class to search for
      * @return List of Strings containing discovered class names.
+     * @throws IOException when the used {@link ClassFinder} throws one while searching for the class
      */
     public static List<String> findClassesThatExtend(Class<?> superClass)
         throws IOException {
@@ -330,6 +334,7 @@ public class JMeterUtils implements UnitTestManager {
      *
      * @param r -
      *            the upper bound (exclusive)
+     * @return a random <code>int</code>
      */
     public static int getRandomInt(int r) {
         return rand.nextInt(r);
@@ -414,6 +419,8 @@ public class JMeterUtils implements UnitTestManager {
         LocaleChangeEvent event = new LocaleChangeEvent(JMeterUtils.class, locale);
         @SuppressWarnings("unchecked") // clone will produce correct type
         // TODO but why do we need to clone the list?
+        // ANS: to avoid possible ConcurrentUpdateException when unsubscribing
+        // Could perhaps avoid need to clone by using a modern concurrent list
         Vector<LocaleChangeListener> listeners = (Vector<LocaleChangeListener>) localeChangeListeners.clone();
         for (LocaleChangeListener listener : listeners) {
             listener.localeChanged(event);
@@ -560,7 +567,7 @@ public class JMeterUtils implements UnitTestManager {
 
     /**
      * This looks for the requested image in the classpath under
-     * org.apache.jmeter.images. <name>
+     * org.apache.jmeter.images.&lt;name&gt;
      *
      * @param name
      *            Description of Parameter
@@ -587,7 +594,7 @@ public class JMeterUtils implements UnitTestManager {
 
     /**
      * This looks for the requested image in the classpath under
-     * org.apache.jmeter.images. <name>, and also sets the description
+     * org.apache.jmeter.images.<em>&lt;name&gt;</em>, and also sets the description
      * of the image, which is useful if the icon is going to be placed
      * on the clipboard.
      *
@@ -701,24 +708,22 @@ public class JMeterUtils implements UnitTestManager {
      */
     // TODO only called by UserParameterXMLParser.getXMLParameters which is a deprecated class
     public static XMLReader getXMLParser() {
-        XMLReader reader = null;
         final String parserName = getPropDefault("xml.parser", // $NON-NLS-1$
                 "org.apache.xerces.parsers.SAXParser");  // $NON-NLS-1$
-        try {
-            reader = (XMLReader) instantiate(parserName,
-                    "org.xml.sax.XMLReader"); // $NON-NLS-1$
-            // reader = xmlFactory.newSAXParser().getXMLReader();
-        } catch (Exception e) {
-            reader = (XMLReader) instantiate(parserName, // $NON-NLS-1$
-                    "org.xml.sax.XMLReader"); // $NON-NLS-1$
-        }
-        return reader;
+        return (XMLReader) instantiate(parserName,
+                "org.xml.sax.XMLReader"); // $NON-NLS-1$
     }
 
     /**
      * Creates the vector of alias strings.
+     * <p>
+     * The properties will be filtered by all values starting with
+     * <code>alias.</code>. The matching entries will be used for the new
+     * {@link Hashtable} while the prefix <code>alias.</code> will be stripped
+     * of the keys.
      *
      * @param properties
+     *            the input values
      * @return The Alias value
      */
     public static Hashtable<String, String> getAlias(Properties properties) {
@@ -750,10 +755,22 @@ public class JMeterUtils implements UnitTestManager {
     /**
      * Creates a table of strings for all the properties that start with a
      * common prefix.
+     * <p>
+     * So if you have {@link Properties} <code>prop</code> with two entries, say
+     * <ul>
+     * <li>this.test</li>
+     * <li>that.something</li>
+     * </ul>
+     * And would call this method with a <code>prefix</code> <em>this</em>, the
+     * result would be a new {@link Hashtable} with one entry, which key would
+     * be <em>test</em>.
      *
-     * @param properties input to search
-     * @param prefix to match against properties
-     * @return a Hashtable where the keys are the original keys with the prefix removed
+     * @param properties
+     *            input to search
+     * @param prefix
+     *            to match against properties
+     * @return a Hashtable where the keys are the original matching keys with
+     *         the prefix removed
      */
     public static Hashtable<String, String> getHashtable(Properties properties, String prefix) {
         Hashtable<String, String> t = new Hashtable<String, String>();
@@ -780,9 +797,9 @@ public class JMeterUtils implements UnitTestManager {
     public static int getPropDefault(String propName, int defaultVal) {
         int ans;
         try {
-            ans = (Integer.valueOf(appProperties.getProperty(propName, Integer.toString(defaultVal)).trim()))
-                    .intValue();
+            ans = Integer.parseInt(appProperties.getProperty(propName, Integer.toString(defaultVal)).trim());
         } catch (Exception e) {
+            log.warn("Unexpected value set for int property:'"+propName+"', defaulting to:"+defaultVal);
             ans = defaultVal;
         }
         return ans;
@@ -806,9 +823,10 @@ public class JMeterUtils implements UnitTestManager {
             } else if (strVal.equalsIgnoreCase("false") || strVal.equalsIgnoreCase("f")) { // $NON-NLS-1$  // $NON-NLS-2$
                 ans = false;
             } else {
-                ans = ((Integer.valueOf(strVal)).intValue() == 1);
+                ans = Integer.parseInt(strVal) == 1;
             }
         } catch (Exception e) {
+            log.warn("Unexpected value set for boolean property:'"+propName+"', defaulting to:"+defaultVal);
             ans = defaultVal;
         }
         return ans;
@@ -826,8 +844,9 @@ public class JMeterUtils implements UnitTestManager {
     public static long getPropDefault(String propName, long defaultVal) {
         long ans;
         try {
-            ans = (Long.valueOf(appProperties.getProperty(propName, Long.toString(defaultVal)).trim())).longValue();
+            ans = Long.parseLong(appProperties.getProperty(propName, Long.toString(defaultVal)).trim());
         } catch (Exception e) {
+            log.warn("Unexpected value set for long property:'"+propName+"', defaulting to:"+defaultVal);
             ans = defaultVal;
         }
         return ans;
@@ -844,12 +863,14 @@ public class JMeterUtils implements UnitTestManager {
      */
     public static String getPropDefault(String propName, String defaultVal) {
         String ans = defaultVal;
-        try {
+        try 
+        {
             String value = appProperties.getProperty(propName, defaultVal);
             if(value != null) {
                 ans = value.trim();
             }
         } catch (Exception e) {
+            // TODO Can this happen ?
             ans = defaultVal;
         }
         return ans;
@@ -867,6 +888,7 @@ public class JMeterUtils implements UnitTestManager {
         try {
             ans = appProperties.getProperty(propName);
         } catch (Exception e) {
+            // TODO Can this happen ?
             ans = null;
         }
         return ans;
@@ -889,6 +911,10 @@ public class JMeterUtils implements UnitTestManager {
      * Sets the selection of the JComboBox to the Object 'name' from the list in
      * namVec.
      * NOTUSED?
+     * @param properties not used at the moment
+     * @param combo {@link JComboBox} to work on
+     * @param namVec List of names, which are displayed in <code>combo</code>
+     * @param name Name, that is to be selected. It has to be in <code>namVec</code>
      */
     public static void selJComboBoxItem(Properties properties, JComboBox combo, Vector<?> namVec, String name) {
         int idx = namVec.indexOf(name);
@@ -1053,7 +1079,7 @@ public class JMeterUtils implements UnitTestManager {
                     titleMsg,
                     JOptionPane.ERROR_MESSAGE);
         } catch (HeadlessException e) {
-                log.warn("reportErrorToUser(\"" + errorMsg + "\") caused", e);
+            log.warn("reportErrorToUser(\"" + errorMsg + "\") caused", e);
         }
     }
 
@@ -1198,7 +1224,7 @@ public class JMeterUtils implements UnitTestManager {
     /**
      * Find a file in the current directory or in the JMeter bin directory.
      *
-     * @param fileName
+     * @param fileName the name of the file to find
      * @return File object
      */
     public static File findFile(String fileName){
@@ -1325,4 +1351,4 @@ public class JMeterUtils implements UnitTestManager {
             // NOOP
         }
     }
-}
\ No newline at end of file
+}
