@@ -96,6 +96,9 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     private static final boolean DEFAULT_CLEAR_VALUE = false;
 
+    /** Decides whether port should be omitted from SPN for kerberos spnego authentication */
+    private static final boolean STRIP_PORT = JMeterUtils.getPropDefault("kerberos.spnego.strip_port", true);
+
     public enum Mechanism {
         BASIC_DIGEST, KERBEROS;
     }
@@ -131,6 +134,21 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     /**
      * Update an authentication record.
+     *
+     * @param index
+     *            index at which position the record should be set
+     * @param url
+     *            url for which the authentication record should be used
+     * @param user
+     *            name of the user
+     * @param pass
+     *            password of the user
+     * @param domain
+     *            domain of the user
+     * @param realm
+     *            realm of the site
+     * @param mechanism
+     *            authentication {@link Mechanism} to use
      */
     public void set(int index, String url, String user, String pass, String domain, String realm, Mechanism mechanism) {
         Authorization auth = new Authorization(url, user, pass, domain, realm, mechanism);
@@ -167,6 +185,10 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     /**
      * Return the record at index i
+     *
+     * @param i
+     *            index of the record to get
+     * @return authorization record at index <code>i</code>
      */
     public Authorization get(int i) {
         return (Authorization) getAuthObjects().get(i).getObjectValue();
@@ -230,14 +252,25 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
     }
 
     /**
-     * @return boolean true if an authorization is setup for url
+     * Tests whether an authorization record is available for a given URL
+     *
+     * @param url
+     *            {@link URL} for which an authorization record should be
+     *            available
+     * @return <code>true</code> if an authorization is setup for url,
+     *         <code>false</code> otherwise
      */
     public boolean hasAuthForURL(URL url) {
         return getAuthForURL(url) != null;
     }
     
     /**
-     * @return Subject if Auth Scheme uses Subject and an authorization is setup for url
+     * Get a {@link Subject} for a given URL, if available
+     *
+     * @param url
+     *            {@link URL} for which the subject was asked
+     * @return Subject if Auth Scheme uses Subject and an authorization is setup
+     *         for <code>url</code>, <code>null</code> otherwise
      */
     public Subject getSubjectForUrl(URL url) {
         Authorization authorization = getAuthForURL(url);
@@ -253,8 +286,32 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
     public void addConfigElement(ConfigElement config) {
     }
 
-    public void addAuth(Authorization auth) {
-        getAuthObjects().addItem(auth);
+    /**
+     * Add newAuthorization if it does not already exist
+     * @param newAuthorization authorization to be added
+     */
+    public void addAuth(Authorization newAuthorization) {
+        boolean alreadyExists=false;
+        PropertyIterator iter = getAuthObjects().iterator();
+        //iterate over authentication objects in manager
+        while (iter.hasNext()) {
+            Authorization authorization = (Authorization) iter.next().getObjectValue();
+            if (authorization == null) {
+                continue;
+            }
+            if (match(authorization,newAuthorization)) {
+                if (log.isDebugEnabled()) {
+                    log.debug("Found the same Authorization object:" + newAuthorization.toString());
+                }
+                //set true, if found the same one
+                alreadyExists=true;
+                break;
+            }
+        }
+        if(!alreadyExists){
+            // if there was no such auth object, add.
+            getAuthObjects().addItem(newAuthorization);
+        }
     }
 
     public void addAuth() {
@@ -269,6 +326,11 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     /**
      * Save the authentication data to a file.
+     *
+     * @param authFile
+     *            path of the file to save the authentication data to
+     * @throws IOException
+     *             when writing to the file fails
      */
     public void save(String authFile) throws IOException {
         File file = new File(authFile);
@@ -292,6 +354,11 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     /**
      * Add authentication data from a file.
+     *
+     * @param authFile
+     *            path to the file to read the authentication data from
+     * @throws IOException
+     *             when reading the data fails
      */
     public void addFile(String authFile) throws IOException {
         File file = new File(authFile);
@@ -343,6 +410,9 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     /**
      * Remove an authentication record.
+     *
+     * @param index
+     *            index of the authentication record to remove
      */
     public void remove(int index) {
         getAuthObjects().remove(index);
@@ -362,6 +432,8 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
 
     /**
      * Return the number of records.
+     *
+     * @return the number of records
      */
     public int getAuthCount() {
         return getAuthObjects().size();
@@ -392,7 +464,7 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
                 log.debug(username + " > D="+domain+" R="+realm + " M="+auth.getMechanism());
             }
             if (Mechanism.KERBEROS.equals(auth.getMechanism())) {
-                ((AbstractHttpClient) client).getAuthSchemes().register(AuthPolicy.SPNEGO, new SPNegoSchemeFactory(true));
+                ((AbstractHttpClient) client).getAuthSchemes().register(AuthPolicy.SPNEGO, new SPNegoSchemeFactory(isStripPort(url)));
                 credentialsProvider.setCredentials(new AuthScope(null, -1, null), USE_JAAS_CREDENTIALS);
             } else {
                 credentialsProvider.setCredentials(
@@ -402,6 +474,38 @@ public class AuthManager extends ConfigTestElement implements TestStateListener,
         }
     }
 
+    /**
+     * IE and Firefox will always strip port from the url before constructing
+     * the SPN. Chrome has an option (<code>--enable-auth-negotiate-port</code>)
+     * to include the port if it differs from <code>80</code> or
+     * <code>443</code>. That behavior can be changed by setting the jmeter
+     * property <code>kerberos.spnego.strip_port</code>.
+     *
+     * @param url to be checked
+     * @return <code>true</code> when port should omitted in SPN
+     */
+    private boolean isStripPort(URL url) {
+        if (STRIP_PORT) {
+            return true;
+        }
+        return (url.getPort() == HTTPConstants.DEFAULT_HTTP_PORT ||
+                url.getPort() == HTTPConstants.DEFAULT_HTTPS_PORT);
+    }
+
+    /**
+     * Check if two authorization objects are equal ignoring username/password
+     * @param a {@link Authorization}
+     * @param b {@link Authorization}
+     * @return true if a and b match
+     */
+    private boolean match(Authorization a, Authorization b){
+        return
+                a.getURL().equals(b.getURL())&&
+                a.getDomain().equals(b.getDomain())&&
+                a.getRealm().equals(b.getRealm())&&
+                a.getMechanism().equals(b.getMechanism());
+    }
+
     /** {@inheritDoc} */
     @Override
     public void testStarted() {
