@@ -24,6 +24,7 @@ import java.io.Serializable;
 import java.net.HttpURLConnection;
 import java.net.URL;
 import java.net.URLConnection;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
 import java.util.Map;
@@ -31,18 +32,19 @@ import java.util.Map;
 import org.apache.commons.collections.map.LRUMap;
 import org.apache.commons.httpclient.HttpMethod;
 import org.apache.commons.httpclient.URIException;
-import org.apache.commons.httpclient.util.DateParseException;
-import org.apache.commons.httpclient.util.DateUtil;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.HttpRequestBase;
+import org.apache.http.impl.cookie.DateParseException;
+import org.apache.http.impl.cookie.DateUtils;
 import org.apache.jmeter.config.ConfigTestElement;
 import org.apache.jmeter.engine.event.LoopIterationEvent;
+import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
 import org.apache.jmeter.protocol.http.util.HTTPConstants;
-import org.apache.jmeter.samplers.SampleResult;
 import org.apache.jmeter.testelement.TestIterationListener;
 import org.apache.jmeter.testelement.TestStateListener;
 import org.apache.jmeter.testelement.property.BooleanProperty;
+import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jorphan.logging.LoggingManager;
 import org.apache.log.Logger;
 
@@ -57,6 +59,12 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
 
     private static final Logger log = LoggingManager.getLoggerForClass();
 
+    private static final String[] CACHEABLE_METHODS = JMeterUtils.getPropDefault("cacheable_methods", "GET").split("[ ,]");
+
+    static {
+        log.info("Will only cache the following methods: "+Arrays.toString(CACHEABLE_METHODS));
+    }
+
     //+ JMX attributes, do not change values
     public static final String CLEAR = "clearEachIteration"; // $NON-NLS-1$
     public static final String USE_EXPIRES = "useExpires"; // $NON-NLS-1$
@@ -113,7 +121,7 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
      * @param conn connection
      * @param res result
      */
-    public void saveDetails(URLConnection conn, SampleResult res){
+    public void saveDetails(URLConnection conn, HTTPSampleResult res){
         if (isCacheable(res)){
             String lastModified = conn.getHeaderField(HTTPConstants.LAST_MODIFIED);
             String expires = conn.getHeaderField(HTTPConstants.EXPIRES);
@@ -126,12 +134,17 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
     }
 
     /**
-     * Save the Last-Modified, Etag, and Expires headers if the result is cacheable.
-     * Version for Commons HttpClient implementation.
+     * Save the Last-Modified, Etag, and Expires headers if the result is
+     * cacheable. Version for Commons HttpClient implementation.
+     *
      * @param method
-     * @param res result
+     *            {@link HttpMethod} to get header information from
+     * @param res
+     *            result to decide if result is cacheable
+     * @throws URIException
+     *             if extraction of the the uri from <code>method</code> fails
      */
-    public void saveDetails(HttpMethod method, SampleResult res) throws URIException{
+    public void saveDetails(HttpMethod method, HTTPSampleResult res) throws URIException{
         if (isCacheable(res)){
             String lastModified = getHeader(method ,HTTPConstants.LAST_MODIFIED);
             String expires = getHeader(method ,HTTPConstants.EXPIRES);
@@ -144,12 +157,15 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
     }
 
     /**
-     * Save the Last-Modified, Etag, and Expires headers if the result is cacheable.
-     * Version for Apache HttpClient implementation.
+     * Save the Last-Modified, Etag, and Expires headers if the result is
+     * cacheable. Version for Apache HttpClient implementation.
+     *
      * @param method
-     * @param res result
+     *            {@link HttpResponse} to extract header information from
+     * @param res
+     *            result to decide if result is cacheable
      */
-    public void saveDetails(HttpResponse method, SampleResult res) {
+    public void saveDetails(HttpResponse method, HTTPSampleResult res) {
         if (isCacheable(res)){
             String lastModified = getHeader(method ,HTTPConstants.LAST_MODIFIED);
             String expires = getHeader(method ,HTTPConstants.EXPIRES);
@@ -183,8 +199,8 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
             }
             if (expires != null) {
                 try {
-                    expiresDate = DateUtil.parseDate(expires);
-                } catch (DateParseException e) {
+                    expiresDate = DateUtils.parseDate(expires);
+                } catch (org.apache.http.impl.cookie.DateParseException e) {
                     if (log.isDebugEnabled()){
                         log.debug("Unable to parse Expires: '"+expires+"' "+e);
                     }
@@ -204,8 +220,8 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
                 } else if(expires==null) { // No max-age && No expires
                     if(!StringUtils.isEmpty(lastModified) && !StringUtils.isEmpty(date)) {
                         try {
-                            Date responseDate = DateUtil.parseDate( date );
-                            Date lastModifiedAsDate = DateUtil.parseDate( lastModified );
+                            Date responseDate = DateUtils.parseDate( date );
+                            Date lastModifiedAsDate = DateUtils.parseDate( lastModified );
                             // see https://developer.mozilla.org/en/HTTP_Caching_FAQ
                             // see http://www.ietf.org/rfc/rfc2616.txt#13.2.4 
                             expiresDate=new Date(System.currentTimeMillis()
@@ -249,18 +265,31 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
 
     /*
      * Is the sample result OK to cache?
-     * i.e is it in the 2xx range?
+     * i.e is it in the 2xx range, and is it a cacheable method?
      */
-    private boolean isCacheable(SampleResult res){
+    private boolean isCacheable(HTTPSampleResult res){
         final String responseCode = res.getResponseCode();
-        return "200".compareTo(responseCode) <= 0  // $NON-NLS-1$
-            && "299".compareTo(responseCode) >= 0; // $NON-NLS-1$
+        return isCacheableMethod(res)
+            && "200".compareTo(responseCode) <= 0  // $NON-NLS-1$
+            && "299".compareTo(responseCode) >= 0;  // $NON-NLS-1$
+    }
+
+    private boolean isCacheableMethod(HTTPSampleResult res) {
+        final String resMethod = res.getHTTPMethod();
+        for(String method : CACHEABLE_METHODS) {
+            if (method.equalsIgnoreCase(resMethod)) {
+                return true;
+            }
+        }
+        return false;
     }
 
     /**
-     * Check the cache, and if there is a match, set the headers:<br/>
-     * If-Modified-Since<br/>
-     * If-None-Match<br/>
+     * Check the cache, and if there is a match, set the headers:
+     * <ul>
+     * <li>If-Modified-Since</li>
+     * <li>If-None-Match</li>
+     * </ul>
      * Commons HttpClient version
      * @param url URL to look up in cache
      * @param method where to set the headers
@@ -283,11 +312,13 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
     }
 
     /**
-     * Check the cache, and if there is a match, set the headers:<br/>
-     * If-Modified-Since<br/>
-     * If-None-Match<br/>
+     * Check the cache, and if there is a match, set the headers:
+     * <ul>
+     * <li>If-Modified-Since</li>
+     * <li>If-None-Match</li>
+     * </ul>
      * Apache HttpClient version.
-     * @param url URL to look up in cache
+     * @param url {@link URL} to look up in cache
      * @param request where to set the headers
      */
     public void setHeaders(URL url, HttpRequestBase request) {
@@ -308,10 +339,12 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
     }
 
     /**
-     * Check the cache, and if there is a match, set the headers:<br/>
-     * If-Modified-Since<br/>
-     * If-None-Match<br/>
-     * @param url URL to look up in cache
+     * Check the cache, and if there is a match, set the headers:
+     * <ul>
+     * <li>If-Modified-Since</li>
+     * <li>If-None-Match</li>
+     * </ul>
+     * @param url {@link URL} to look up in cache
      * @param conn where to set the headers
      */
     public void setHeaders(HttpURLConnection conn, URL url) {
@@ -332,8 +365,9 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
     }
 
     /**
-     * Check the cache, if the entry has an expires header and the entry has not expired, return true<br/>
-     * @param url URL to look up in cache
+     * Check the cache, if the entry has an expires header and the entry has not expired, return true<br>
+     * @param url {@link URL} to look up in cache
+     * @return <code>true</code> if entry has an expires header and the entry has not expired, else <code>false</code>
      */
     public boolean inCache(URL url) {
         CacheEntry entry = getCache().get(url.toString());
@@ -436,4 +470,4 @@ public class CacheManager extends ConfigTestElement implements TestStateListener
         useExpires=getUseExpires(); // cache the value
     }
 
-}
\ No newline at end of file
+}
