@@ -114,9 +114,9 @@ public class DefaultSamplerCreator extends AbstractSamplerCreator {
      * Compute sampler informations from Request Header
      * @param sampler {@link HTTPSamplerBase}
      * @param request {@link HttpRequestHdr}
-     * @param pageEncodings Map<String, String>
-     * @param formEncodings Map<String, String>
-     * @throws Exception
+     * @param pageEncodings Map of page encodings
+     * @param formEncodings Map of form encodings
+     * @throws Exception when something fails
      */
     protected void computeFromHeader(HTTPSamplerBase sampler,
             HttpRequestHdr request, Map<String, String> pageEncodings,
@@ -141,7 +141,7 @@ public class DefaultSamplerCreator extends AbstractSamplerCreator {
      * Compute sampler informations from Request Header
      * @param sampler {@link HTTPSamplerBase}
      * @param request {@link HttpRequestHdr}
-     * @throws Exception
+     * @throws Exception when something fails
      */
     protected void computeFromPostBody(HTTPSamplerBase sampler,
             HttpRequestHdr request) throws Exception {
@@ -187,6 +187,7 @@ public class DefaultSamplerCreator extends AbstractSamplerCreator {
                 sampler.setArguments(urlConfig.getArguments());
                 // Set the file uploads
                 sampler.setHTTPFiles(urlConfig.getHTTPFileArgs().asArray());
+                sampler.setDoBrowserCompatibleMultipart(true); // we are parsing browser input here
             // used when postData is pure xml (eg. an xml-rpc call) or for PUT
             } else if (postData.trim().startsWith("<?") 
                     || HTTPConstants.PUT.equals(sampler.getMethod())
@@ -310,9 +311,10 @@ public class DefaultSamplerCreator extends AbstractSamplerCreator {
      * Compute content encoding
      * @param sampler {@link HTTPSamplerBase}
      * @param request {@link HttpRequestHdr}
-     * @param pageEncodings Map<String, String>
-     * @param formEncodings Map<String, String>
-     * @throws MalformedURLException
+     * @param pageEncodings Map of page encodings
+     * @param formEncodings Map of form encodings
+     * @throws MalformedURLException when no {@link URL} could be built from
+     *         <code>sampler</code> and <code>request</code>
      */
     protected void computeContentEncoding(HTTPSamplerBase sampler,
             HttpRequestHdr request, Map<String, String> pageEncodings,
@@ -341,8 +343,9 @@ public class DefaultSamplerCreator extends AbstractSamplerCreator {
      * Computes content encoding from request and if not found uses pageEncoding 
      * and formEncoding to see if URL was previously computed with a content type
      * @param request {@link HttpRequestHdr}
-     * @param pageEncodings Map<String, String>
-     * @param formEncodings Map<String, String>
+     * @param pageEncodings Map of page encodings
+     * @param formEncodings Map of form encodings
+     * @param urlWithoutQuery the request URL without the query parameters
      * @return String content encoding
      */
     protected String computeContentEncoding(HttpRequestHdr request,
@@ -419,4 +422,4 @@ public class DefaultSamplerCreator extends AbstractSamplerCreator {
             log.debug("Proxy: setting server: " + sampler.getDomain());
         }
     }
-}
\ No newline at end of file
+}
