@@ -32,7 +32,9 @@ import java.nio.charset.IllegalCharsetNameException;
 import java.security.GeneralSecurityException;
 import java.security.KeyStore;
 import java.security.KeyStoreException;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
 import javax.net.ssl.KeyManager;
@@ -82,7 +84,7 @@ public class Proxy extends Thread {
         JMeterUtils.getPropDefault("proxy.cert.factory", "SunX509"); // $NON-NLS-1$ $NON-NLS-2$
 
     private static final String SSLCONTEXT_PROTOCOL =
-        JMeterUtils.getPropDefault("proxy.ssl.protocol", "SSLv3"); // $NON-NLS-1$ $NON-NLS-2$
+        JMeterUtils.getPropDefault("proxy.ssl.protocol", "TLS"); // $NON-NLS-1$ $NON-NLS-2$
 
     // HashMap to save ssl connection between Jmeter proxy and browser
     private static final HashMap<String, SSLSocketFactory> HOST2SSL_SOCK_FAC = new HashMap<String, SSLSocketFactory>();
@@ -137,7 +139,7 @@ public class Proxy extends Thread {
      *            the ProxyControl which will receive the generated sampler
      * @param _pageEncodings
      *            reference to the Map of Deamon, with mappings from page urls to encoding used
-     * @param formEncodingsEncodings
+     * @param _formEncodings
      *            reference to the Map of Deamon, with mappings from form action urls to encoding used
      */
     void configure(Socket _clientSocket, ProxyControl _target, Map<String, String> _pageEncodings, Map<String, String> _formEncodings) {
@@ -167,6 +169,7 @@ public class Proxy extends Thread {
         if (isDebug) {
             log.debug(port + "====================================================================");
         }
+        SamplerCreator samplerCreator = null;
         try {
             // Now, parse initial request (in case it is a CONNECT request)
             byte[] ba = request.parse(new BufferedInputStream(clientSocket.getInputStream()));
@@ -204,7 +207,8 @@ public class Proxy extends Thread {
                 try {
                     ba = request.parse(new BufferedInputStream(clientSocket.getInputStream()));
                 } catch (IOException ioe) { // most likely this is because of a certificate error
-                    final String url = param.length>0 ?  " for '"+ param[0] +"'" : "";
+                    // param.length is 2 here
+                    final String url = " for '"+ param[0] +"'";
                     log.warn(port + "Problem with SSL certificate"+url+"? Ensure browser is set to accept the JMeter proxy cert: " + ioe.getMessage());
                     // won't work: writeErrorToClient(HttpReplyHdr.formInternalError());
                     result = generateErrorResult(result, request, ioe, "\n**ensure browser is set to accept the JMeter proxy certificate**"); // Generate result (if nec.) and populate it
@@ -219,7 +223,7 @@ public class Proxy extends Thread {
                 }
             }
 
-            SamplerCreator samplerCreator = SAMPLERFACTORY.getSamplerCreator(request, pageEncodings, formEncodings);
+            samplerCreator = SAMPLERFACTORY.getSamplerCreator(request, pageEncodings, formEncodings);
             sampler = samplerCreator.createAndPopulateSampler(request, pageEncodings, formEncodings);
 
             /*
@@ -267,7 +271,9 @@ public class Proxy extends Thread {
              */
             if (headers != null) {
                 headers.removeHeaderNamed(HTTPConstants.HEADER_COOKIE);// Always remove cookies
-                headers.removeHeaderNamed(HTTPConstants.HEADER_AUTHORIZATION);// Always remove authorization
+                // See https://issues.apache.org/bugzilla/show_bug.cgi?id=25430
+                // HEADER_AUTHORIZATION won't be removed, it will be used
+                // for creating Authorization Manager
                 // Remove additional headers
                 for(String hdr : HEADERS_TO_REMOVE){
                     headers.removeHeaderNamed(hdr);
@@ -275,7 +281,17 @@ public class Proxy extends Thread {
             }
             if(result != null) // deliverSampler allows sampler to be null, but result must not be null
             {
-                target.deliverSampler(sampler, new TestElement[] { captureHttpHeaders ? headers : null }, result);
+                List<TestElement> children = new ArrayList<TestElement>();
+                if(captureHttpHeaders) {
+                    children.add(headers);
+                }
+                if(samplerCreator != null) {
+                    children.addAll(samplerCreator.createChildren(sampler, result));
+                } 
+                target.deliverSampler(sampler,
+                        children.isEmpty() ? null : (TestElement[]) children
+                                .toArray(new TestElement[children.size()]),
+                        result);
             }
             try {
                 clientSocket.close();
