@@ -19,6 +19,7 @@
 package org.apache.jmeter.protocol.http.proxy.gui;
 
 import java.awt.BorderLayout;
+import java.awt.Component;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.datatransfer.DataFlavor;
@@ -45,6 +46,8 @@ import javax.swing.JComboBox;
 import javax.swing.JLabel;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
+import javax.swing.JPopupMenu;
+import javax.swing.JScrollBar;
 import javax.swing.JScrollPane;
 import javax.swing.JTable;
 import javax.swing.JTextField;
@@ -63,6 +66,7 @@ import org.apache.jmeter.gui.util.HorizontalPanel;
 import org.apache.jmeter.gui.util.MenuFactory;
 import org.apache.jmeter.gui.util.PowerTableModel;
 import org.apache.jmeter.gui.util.VerticalPanel;
+import org.apache.jmeter.protocol.http.control.RecordingController;
 import org.apache.jmeter.protocol.http.proxy.ProxyControl;
 import org.apache.jmeter.protocol.http.sampler.HTTPSamplerFactory;
 import org.apache.jmeter.testelement.TestElement;
@@ -160,6 +164,11 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
      * List of available target controllers
      */
     private JComboBox targetNodes;
+    
+    /**
+     * Notify child Listener of Filtered Samplers
+     */
+    private JCheckBox notifyChildSamplerListenerOfFilteredSamplersCB;
 
     private DefaultComboBoxModel targetNodesModel;
 
@@ -254,6 +263,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
             model.setSamplerFollowRedirects(samplerFollowRedirects.isSelected());
             model.setUseKeepAlive(useKeepAlive.isSelected());
             model.setSamplerDownloadImages(samplerDownloadImages.isSelected());
+            model.setNotifyChildSamplerListenerOfFilteredSamplers(notifyChildSamplerListenerOfFilteredSamplersCB.isSelected());
             model.setRegexMatch(regexMatch.isSelected());
             model.setContentTypeInclude(contentTypeInclude.getText());
             model.setContentTypeExclude(contentTypeExclude.getText());
@@ -313,6 +323,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         samplerFollowRedirects.setSelected(model.getSamplerFollowRedirects());
         useKeepAlive.setSelected(model.getUseKeepalive());
         samplerDownloadImages.setSelected(model.getSamplerDownloadImages());
+        notifyChildSamplerListenerOfFilteredSamplersCB.setSelected(model.getNotifyChildSamplerListenerOfFilteredSamplers());
         regexMatch.setSelected(model.getRegexMatch());
         contentTypeInclude.setText(model.getContentTypeInclude());
         contentTypeExclude.setText(model.getContentTypeExclude());
@@ -476,6 +487,17 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
     private void startProxy() {
         ValueReplacer replacer = GuiPackage.getInstance().getReplacer();
         modifyTestElement(model);
+        TreeNodeWrapper treeNodeWrapper = (TreeNodeWrapper)targetNodesModel.getSelectedItem();
+        if (JMeterUtils.getResString("use_recording_controller").equals(treeNodeWrapper.getLabel())) {
+            JMeterTreeNode targetNode = model.findTargetControllerNode();
+            if(targetNode == null || !(targetNode.getTestElement() instanceof RecordingController)) {
+                JOptionPane.showMessageDialog(this,
+                        JMeterUtils.getResString("proxy_cl_wrong_target_cl"), // $NON-NLS-1$
+                        JMeterUtils.getResString("error_title"), // $NON-NLS-1$
+                        JOptionPane.ERROR_MESSAGE);
+                return;
+            }
+        }
         // Proxy can take some while to start up; show a wating cursor
         Cursor cursor = getCursor();
         setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
@@ -589,6 +611,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         Box includeExcludePanel = Box.createVerticalBox();
         includeExcludePanel.add(createIncludePanel());
         includeExcludePanel.add(createExcludePanel());
+        includeExcludePanel.add(createNotifyListenersPanel());
         mainPanel.add(includeExcludePanel, BorderLayout.CENTER);
 
         mainPanel.add(createControls(), BorderLayout.SOUTH);
@@ -732,6 +755,13 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
     private JPanel createTargetPanel() {
         targetNodesModel = new DefaultComboBoxModel();
         targetNodes = new JComboBox(targetNodesModel);
+        targetNodes.setPrototypeDisplayValue(""); // $NON-NLS-1$ // Bug 56303 fixed the width of combo list
+        JPopupMenu popup = (JPopupMenu) targetNodes.getUI().getAccessibleChild(targetNodes, 0); // get popup element
+        JScrollPane scrollPane = findScrollPane(popup);
+        if(scrollPane != null) {
+            scrollPane.setHorizontalScrollBar(new JScrollBar(JScrollBar.HORIZONTAL)); // add scroll pane if label element is too long
+            scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
+        }
         targetNodes.setActionCommand(CHANGE_TARGET);
         // Action listener will be added later
 
@@ -745,6 +775,16 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         return panel;
     }
 
+    private JScrollPane findScrollPane(JPopupMenu popup) {
+        Component[] components = popup.getComponents();
+        for (Component component : components) {
+            if(component instanceof JScrollPane) {
+                return (JScrollPane) component;
+            }
+        }
+        return null;
+    }
+
     private JPanel createGroupingPanel() {
         DefaultComboBoxModel m = new DefaultComboBoxModel();
         // Note: position of these elements in the menu *must* match the
@@ -829,6 +869,20 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         return panel;
     }
 
+    private JPanel createNotifyListenersPanel() {
+        JPanel panel = new JPanel();
+        panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), JMeterUtils
+                .getResString("notify_child_listeners_fr"))); // $NON-NLS-1$
+        
+        notifyChildSamplerListenerOfFilteredSamplersCB = new JCheckBox(JMeterUtils.getResString("notify_child_listeners_fr")); // $NON-NLS-1$
+        notifyChildSamplerListenerOfFilteredSamplersCB.setSelected(true);
+        notifyChildSamplerListenerOfFilteredSamplersCB.addActionListener(this);
+        notifyChildSamplerListenerOfFilteredSamplersCB.setActionCommand(ENABLE_RESTART);
+
+        panel.add(notifyChildSamplerListenerOfFilteredSamplersCB);
+        return panel;
+    }
+
     private JPanel createTableButtonPanel(String addCommand, String deleteCommand, String copyFromClipboard, String addSuggestedExcludes) {
         JPanel buttonPanel = new JPanel();
 
@@ -877,8 +931,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         for (int i = 0; i < targetNodesModel.getSize(); i++) {
             choice = (TreeNodeWrapper) targetNodesModel.getElementAt(i);
             log.debug("Selecting item " + choice + " for model " + model + " in " + this);
-            if (choice.getTreeNode() == model.getTarget()) // .equals caused
-                                                            // NPE
+            if (choice.getTreeNode() == model.getTarget()) // .equals caused NPE
             {
                 break;
             }
@@ -923,4 +976,4 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
             }
         }
     }
-}
\ No newline at end of file
+}
