@@ -69,7 +69,6 @@ import org.apache.jmeter.protocol.http.control.Authorization;
 import org.apache.jmeter.protocol.http.control.CacheManager;
 import org.apache.jmeter.protocol.http.control.CookieManager;
 import org.apache.jmeter.protocol.http.control.HeaderManager;
-import org.apache.jmeter.protocol.http.util.ConversionUtils;
 import org.apache.jmeter.protocol.http.util.EncoderCache;
 import org.apache.jmeter.protocol.http.util.HTTPArgument;
 import org.apache.jmeter.protocol.http.util.HTTPConstants;
@@ -95,8 +94,6 @@ public class HTTPHC3Impl extends HTTPHCAbstractImpl {
     /** retry count to be used (default 1); 0 = disable retries */
     private static final int RETRY_COUNT = JMeterUtils.getPropDefault("httpclient3.retrycount", 0);
 
-    private static final boolean STRICT_RFC_2616 = JMeterUtils.getPropDefault("jmeter.httpclient.strict_rfc2616", false);
-
     private static final String HTTP_AUTHENTICATION_PREEMPTIVE = "http.authentication.preemptive"; // $NON-NLS-1$
 
     private static final boolean canSetPreEmptive; // OK to set pre-emptive auth?
@@ -194,8 +191,10 @@ public class HTTPHC3Impl extends HTTPHCAbstractImpl {
 
         String urlStr = url.toString();
 
-        log.debug("Start : sample " + urlStr);
-        log.debug("method " + method);
+        if (log.isDebugEnabled()) {
+            log.debug("Start : sample " + urlStr);
+            log.debug("method " + method+ " followingRedirect " + areFollowingRedirect + " depth " + frameDepth);            
+        }
 
         HttpMethodBase httpMethod = null;
 
@@ -242,10 +241,7 @@ public class HTTPHC3Impl extends HTTPHCAbstractImpl {
             final CacheManager cacheManager = getCacheManager();
             if (cacheManager != null && HTTPConstants.GET.equalsIgnoreCase(method)) {
                if (cacheManager.inCache(url)) {
-                   res.sampleEnd();
-                   res.setResponseNoContent();
-                   res.setSuccessful(true);
-                   return res;
+                   return updateSampleResultForResourceInCache(res);
                }
             }
 
@@ -322,16 +318,8 @@ public class HTTPHC3Impl extends HTTPHCAbstractImpl {
                 if (headerLocation == null) { // HTTP protocol violation, but avoids NPE
                     throw new IllegalArgumentException("Missing location header");
                 }
-                try {
-                    String redirectLocation = headerLocation.getValue();
-                    if(!STRICT_RFC_2616 && !(redirectLocation.startsWith("http://")||redirectLocation.startsWith("https://"))) {
-                        redirectLocation = ConversionUtils.buildFullUrlFromRelative(url, redirectLocation);
-                    }
-
-                    res.setRedirectLocation(ConversionUtils.sanitizeUrl(new URL(redirectLocation)).toString());
-                } catch (Exception e) {
-                    log.error("Error sanitizing URL:"+headerLocation.getValue()+", message:"+e.getMessage());
-                }
+                String redirectLocation = headerLocation.getValue();
+                res.setRedirectLocation(redirectLocation); // in case sanitising fails
             }
 
             // record some sizes to allow HTTPSampleResult.getBytes() with different options
@@ -374,6 +362,8 @@ public class HTTPHC3Impl extends HTTPHCAbstractImpl {
         } catch (IOException e) {
             res.sampleEnd();
             // pick up headers if failed to execute the request
+            // httpMethod cannot be null here, otherwise 
+            // it would have been caught in the previous catch block
             res.setRequestHeaders(getConnectionHeaders(httpMethod));
             errorResult(e, res);
             return res;
