@@ -36,6 +36,7 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.TimeUnit;
 
 import javax.security.auth.Subject;
 
@@ -73,6 +74,9 @@ import org.apache.http.client.methods.HttpUriRequest;
 import org.apache.http.client.params.ClientPNames;
 import org.apache.http.client.params.CookiePolicy;
 import org.apache.http.client.protocol.ResponseContentEncoding;
+import org.apache.http.conn.ClientConnectionManager;
+import org.apache.http.conn.ConnectionKeepAliveStrategy;
+import org.apache.http.conn.DnsResolver;
 import org.apache.http.conn.params.ConnRoutePNames;
 import org.apache.http.conn.scheme.Scheme;
 import org.apache.http.conn.scheme.SchemeRegistry;
@@ -85,8 +89,11 @@ import org.apache.http.entity.mime.MultipartEntity;
 import org.apache.http.entity.mime.content.FileBody;
 import org.apache.http.entity.mime.content.StringBody;
 import org.apache.http.impl.client.AbstractHttpClient;
+import org.apache.http.impl.client.DefaultConnectionKeepAliveStrategy;
 import org.apache.http.impl.client.DefaultHttpClient;
 import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
+import org.apache.http.impl.conn.SchemeRegistryFactory;
+import org.apache.http.impl.conn.SystemDefaultDnsResolver;
 import org.apache.http.message.BasicNameValuePair;
 import org.apache.http.params.BasicHttpParams;
 import org.apache.http.params.CoreConnectionPNames;
@@ -102,7 +109,6 @@ import org.apache.jmeter.protocol.http.control.AuthManager;
 import org.apache.jmeter.protocol.http.control.CacheManager;
 import org.apache.jmeter.protocol.http.control.CookieManager;
 import org.apache.jmeter.protocol.http.control.HeaderManager;
-import org.apache.jmeter.protocol.http.util.ConversionUtils;
 import org.apache.jmeter.protocol.http.util.EncoderCache;
 import org.apache.jmeter.protocol.http.util.HC4TrustAllSSLSocketFactory;
 import org.apache.jmeter.protocol.http.util.HTTPArgument;
@@ -127,13 +133,33 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
 
     private static final Logger log = LoggingManager.getLoggerForClass();
 
-    private static final boolean STRICT_RFC_2616 = JMeterUtils.getPropDefault("jmeter.httpclient.strict_rfc2616", false);
-
-    /** retry count to be used (default 1); 0 = disable retries */
+    /** retry count to be used (default 0); 0 = disable retries */
     private static final int RETRY_COUNT = JMeterUtils.getPropDefault("httpclient4.retrycount", 0);
 
-    private static final String CONTEXT_METRICS = "jmeter_metrics"; // TODO hack, to be removed later
+    /** Idle timeout to be applied to connections if no Keep-Alive header is sent by the server (default 0 = disable) */
+    private static final int IDLE_TIMEOUT = JMeterUtils.getPropDefault("httpclient4.idletimeout", 0);
 
+    private static final String CONTEXT_METRICS = "jmeter_metrics"; // TODO hack for metrics related to HTTPCLIENT-1081, to be removed later
+
+    private static final ConnectionKeepAliveStrategy IDLE_STRATEGY = new DefaultConnectionKeepAliveStrategy(){
+        @Override
+        public long getKeepAliveDuration(HttpResponse response, HttpContext context) {
+            long duration = super.getKeepAliveDuration(response, context);
+            if (duration <= 0) {// none found by the superclass
+                log.debug("Setting keepalive to " + IDLE_TIMEOUT);
+                return IDLE_TIMEOUT;
+            }
+            return duration; // return the super-class value
+        }
+        
+    };
+
+    /**
+     * Special interceptor made to keep metrics when connection is released for some method like HEAD
+     * Otherwise calling directly ((HttpConnection) localContext.getAttribute(ExecutionContext.HTTP_CONNECTION)).getMetrics();
+     * would throw org.apache.http.impl.conn.ConnectionShutdownException
+     * See https://issues.apache.org/jira/browse/HTTPCLIENT-1081
+     */
     private static final HttpResponseInterceptor METRICS_SAVER = new HttpResponseInterceptor(){
         @Override
         public void process(HttpResponse response, HttpContext context)
@@ -153,7 +179,10 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
         }
     };
 
-    private static final ThreadLocal<Map<HttpClientKey, HttpClient>> HTTPCLIENTS = 
+    /**
+     * 1 HttpClient instance per combination of (HttpClient,HttpClientKey)
+     */
+    private static final ThreadLocal<Map<HttpClientKey, HttpClient>> HTTPCLIENTS_CACHE_PER_THREAD_AND_HTTPCLIENTKEY = 
         new ThreadLocal<Map<HttpClientKey, HttpClient>>(){
         @Override
         protected Map<HttpClientKey, HttpClient> initialValue() {
@@ -177,7 +206,7 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
         log.info("HTTP request retry count = "+RETRY_COUNT);
         
         DEFAULT_HTTP_PARAMS = new SyncBasicHttpParams(); // Could we drop the Sync here?
-        DEFAULT_HTTP_PARAMS.setBooleanParameter("http.connection.stalecheck", false);
+        DEFAULT_HTTP_PARAMS.setBooleanParameter(CoreConnectionPNames.STALE_CONNECTION_CHECK, false);
         DefaultHttpClient.setDefaultHttpParams(DEFAULT_HTTP_PARAMS);
         
         // Process Apache HttpClient parameters file
@@ -243,10 +272,15 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
     protected HTTPSampleResult sample(URL url, String method,
             boolean areFollowingRedirect, int frameDepth) {
 
+        if (log.isDebugEnabled()) {
+            log.debug("Start : sample " + url.toString());
+            log.debug("method " + method+ " followingRedirect " + areFollowingRedirect + " depth " + frameDepth);            
+        }
+
         HTTPSampleResult res = createSampleResult(url, method);
 
-        HttpClient httpClient = setupClient(url);
-        
+        HttpClient httpClient = setupClient(url, res);
+
         HttpRequestBase httpRequest = null;
         try {
             URI uri = url.toURI();
@@ -266,6 +300,8 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
                 httpRequest = new HttpGet(uri);
             } else if (method.equals(HTTPConstants.PATCH)) {
                 httpRequest = new HttpPatch(uri);
+            } else if (HttpWebdav.isWebdavMethod(method)) {
+                httpRequest = new HttpWebdav(method, uri);
             } else {
                 throw new IllegalArgumentException("Unexpected method: '"+method+"'");
             }
@@ -284,10 +320,7 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
         final CacheManager cacheManager = getCacheManager();
         if (cacheManager != null && HTTPConstants.GET.equalsIgnoreCase(method)) {
            if (cacheManager.inCache(url)) {
-               res.sampleEnd();
-               res.setResponseNoContent();
-               res.setSuccessful(true);
-               return res;
+               return updateSampleResultForResourceInCache(res);
            }
         }
 
@@ -336,16 +369,7 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
                     throw new IllegalArgumentException("Missing location header in redirect for " + httpRequest.getRequestLine());
                 }
                 String redirectLocation = headerLocation.getValue();
-                if(!STRICT_RFC_2616 && !(redirectLocation.startsWith("http://")|| redirectLocation.startsWith("https://"))) {
-                    redirectLocation = ConversionUtils.buildFullUrlFromRelative(url, redirectLocation);
-                }
-                try {
-                    final URL redirectUrl = new URL(redirectLocation);
-                    res.setRedirectLocation(ConversionUtils.sanitizeUrl(redirectUrl).toString());
-                } catch (Exception e) {
-                    log.error("Error in redirect URL for "  + httpRequest.getRequestLine()
-                            +"\n\tCould not sanitize URL: " + redirectLocation + "\n\t", e);
-                }
+                res.setRedirectLocation(redirectLocation);
             }
 
             // record some sizes to allow HTTPSampleResult.getBytes() with different options
@@ -388,12 +412,14 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
 
         } catch (IOException e) {
             res.sampleEnd();
+            log.debug("IOException", e);
            // pick up headers if failed to execute the request
             res.setRequestHeaders(getConnectionHeaders((HttpRequest) localContext.getAttribute(ExecutionContext.HTTP_REQUEST)));
             errorResult(e, res);
             return res;
         } catch (RuntimeException e) {
             res.sampleEnd();
+            log.debug("RuntimeException", e);
             errorResult(e, res);
             return res;
         } finally {
@@ -403,13 +429,22 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
     }
 
     /**
-     * Calls sendPostData if method is POST and sendEntityData if method is PUT or PATCH
+     * Calls {@link #sendPostData(HttpPost)} if method is <code>POST</code> and
+     * {@link #sendEntityData(HttpEntityEnclosingRequestBase)} if method is
+     * <code>PUT</code> or <code>PATCH</code>
+     * <p>
      * Field HTTPSampleResult#queryString of result is modified in the 2 cases
-     * @param method String HTTP method
-     * @param result {@link HTTPSampleResult}
-     * @param httpRequest {@link HttpRequestBase}
-     * @param localContext {@link HttpContext}
+     * 
+     * @param method
+     *            String HTTP method
+     * @param result
+     *            {@link HTTPSampleResult}
+     * @param httpRequest
+     *            {@link HttpRequestBase}
+     * @param localContext
+     *            {@link HttpContext}
      * @throws IOException
+     *             when posting data fails due to I/O
      */
     protected void handleMethod(String method, HTTPSampleResult result,
             HttpRequestBase httpRequest, HttpContext localContext) throws IOException {
@@ -418,6 +453,7 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
             String postBody = sendPostData((HttpPost)httpRequest);
             result.setQueryString(postBody);
         } else if (method.equals(HTTPConstants.PUT) || method.equals(HTTPConstants.PATCH)
+                || HttpWebdav.isWebdavMethod(method)
                 || method.equals(HTTPConstants.DELETE)) {
             String entityBody = sendEntityData(( HttpEntityEnclosingRequestBase)httpRequest);
             result.setQueryString(entityBody);
@@ -506,7 +542,7 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
          */
         public HttpClientKey(URL url, boolean hasProxy, String proxyHost,
                 int proxyPort, String proxyUser, String proxyPass) {
-            // N.B. need to separate protocol from authority otherwise http://server would match https://erver
+            // N.B. need to separate protocol from authority otherwise http://server would match https://erver (<= sic, not typo error)
             // could use separate fields, but simpler to combine them
             this.target = url.getProtocol()+"://"+url.getAuthority();
             this.hasProxy = hasProxy;
@@ -563,11 +599,27 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
         public int hashCode(){
             return hashCode;
         }
+
+        // For debugging
+        @Override
+        public String toString() {
+            StringBuilder sb = new StringBuilder();
+            sb.append(target);
+            if (hasProxy) {
+                sb.append(" via ");
+                sb.append(proxyUser);
+                sb.append("@");
+                sb.append(proxyHost);
+                sb.append(":");
+                sb.append(proxyPort);
+            }
+            return sb.toString();
+        }
     }
 
-    private HttpClient setupClient(URL url) {
+    private HttpClient setupClient(URL url, SampleResult res) {
 
-        Map<HttpClientKey, HttpClient> map = HTTPCLIENTS.get();
+        Map<HttpClientKey, HttpClient> mapHttpClientPerHttpClientKey = HTTPCLIENTS_CACHE_PER_THREAD_AND_HTTPCLIENTKEY.get();
         
         final String host = url.getHost();
         final String proxyHost = getProxyHost();
@@ -583,12 +635,12 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
                 useDynamicProxy ? getProxyUser() : PROXY_USER,
                 useDynamicProxy ? getProxyPass() : PROXY_PASS);
         
-        HttpClient httpClient = map.get(key);
+        HttpClient httpClient = mapHttpClientPerHttpClientKey.get(key);
 
         if (httpClient != null && resetSSLContext && HTTPConstants.PROTOCOL_HTTPS.equalsIgnoreCase(url.getProtocol())) {
             ((AbstractHttpClient) httpClient).clearRequestInterceptors(); 
             ((AbstractHttpClient) httpClient).clearResponseInterceptors(); 
-            httpClient.getConnectionManager().shutdown();
+            httpClient.getConnectionManager().closeIdleConnections(1L, TimeUnit.MICROSECONDS);
             httpClient = null;
             JsseSSLManager sslMgr = (JsseSSLManager) SSLManager.getInstance();
             sslMgr.resetContext();
@@ -598,13 +650,22 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
         if (httpClient == null){ // One-time init for this client
 
             HttpParams clientParams = new DefaultedHttpParams(new BasicHttpParams(), DEFAULT_HTTP_PARAMS);
-            
-            httpClient = new DefaultHttpClient(clientParams){
+
+            DnsResolver resolver = this.testElement.getDNSResolver();
+            if (resolver == null) {
+                resolver = new SystemDefaultDnsResolver();
+            }
+            ClientConnectionManager connManager = new MeasuringConnectionManager(SchemeRegistryFactory.createDefault(), resolver);
+
+            httpClient = new DefaultHttpClient(connManager, clientParams) {
                 @Override
                 protected HttpRequestRetryHandler createHttpRequestRetryHandler() {
                     return new DefaultHttpRequestRetryHandler(RETRY_COUNT, false); // set retry count
                 }
             };
+            if (IDLE_TIMEOUT > 0) {
+                ((AbstractHttpClient) httpClient).setKeepAliveStrategy(IDLE_STRATEGY );
+            }
             ((AbstractHttpClient) httpClient).addResponseInterceptor(new ResponseContentEncoding());
             ((AbstractHttpClient) httpClient).addResponseInterceptor(METRICS_SAVER); // HACK
             ((AbstractHttpClient) httpClient).addRequestInterceptor(METRICS_RESETTER); 
@@ -645,16 +706,19 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
             clientParams.setParameter(ClientPNames.COOKIE_POLICY, CookiePolicy.IGNORE_COOKIES);
 
             if (log.isDebugEnabled()) {
-                log.debug("Created new HttpClient: @"+System.identityHashCode(httpClient));
+                log.debug("Created new HttpClient: @"+System.identityHashCode(httpClient) + " " + key.toString());
             }
 
-            map.put(key, httpClient); // save the agent for next time round
+            mapHttpClientPerHttpClientKey.put(key, httpClient); // save the agent for next time round
         } else {
             if (log.isDebugEnabled()) {
-                log.debug("Reusing the HttpClient: @"+System.identityHashCode(httpClient));
+                log.debug("Reusing the HttpClient: @"+System.identityHashCode(httpClient) + " " + key.toString());
             }
         }
 
+        MeasuringConnectionManager connectionManager = (MeasuringConnectionManager) httpClient.getConnectionManager();
+        connectionManager.setSample(res);
+
         // TODO - should this be done when the client is created?
         // If so, then the details need to be added as part of HttpClientKey
         setConnectionAuthorization(httpClient, url, getAuthManager(), key);
@@ -672,10 +736,15 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
      * <li>Calls setConnectionHeaders to setup headers</li>
      * <li>Calls setConnectionCookie to setup Cookie</li>
      * </ul>
+     * 
      * @param url
+     *            {@link URL} of the request
      * @param httpRequest
+     *            http request for the request
      * @param res
+     *            sample result to set cookies on
      * @throws IOException
+     *             if hostname/ip to use could not be figured out
      */
     protected void setupRequest(URL url, HttpRequestBase httpRequest, HTTPSampleResult res)
         throws IOException {
@@ -833,20 +902,23 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
      * @return the headers as a string
      */
     private String getConnectionHeaders(HttpRequest method) {
-        // Get all the request headers
-        StringBuilder hdrs = new StringBuilder(100);
-        Header[] requestHeaders = method.getAllHeaders();
-        for(int i = 0; i < requestHeaders.length; i++) {
-            // Exclude the COOKIE header, since cookie is reported separately in the sample
-            if(!HTTPConstants.HEADER_COOKIE.equalsIgnoreCase(requestHeaders[i].getName())) {
-                hdrs.append(requestHeaders[i].getName());
-                hdrs.append(": "); // $NON-NLS-1$
-                hdrs.append(requestHeaders[i].getValue());
-                hdrs.append("\n"); // $NON-NLS-1$
+        if(method != null) {
+            // Get all the request headers
+            StringBuilder hdrs = new StringBuilder(100);
+            Header[] requestHeaders = method.getAllHeaders();
+            for(int i = 0; i < requestHeaders.length; i++) {
+                // Exclude the COOKIE header, since cookie is reported separately in the sample
+                if(!HTTPConstants.HEADER_COOKIE.equalsIgnoreCase(requestHeaders[i].getName())) {
+                    hdrs.append(requestHeaders[i].getName());
+                    hdrs.append(": "); // $NON-NLS-1$
+                    hdrs.append(requestHeaders[i].getValue());
+                    hdrs.append("\n"); // $NON-NLS-1$
+                }
             }
+    
+            return hdrs.toString();
         }
-
-        return hdrs.toString();
+        return ""; ////$NON-NLS-1$
     }
 
     /**
@@ -903,7 +975,7 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
      * 
      * @param post {@link HttpPost}
      * @return String posted body if computable
-     * @throws IOException
+     * @throws IOException if sending the data fails due to I/O
      */
     protected String sendPostData(HttpPost post)  throws IOException {
         // Buffer to hold the post body, except file content
@@ -1253,14 +1325,14 @@ public class HTTPHC4Impl extends HTTPHCAbstractImpl {
      */
     private void closeThreadLocalConnections() {
         // Does not need to be synchronised, as all access is from same thread
-        Map<HttpClientKey, HttpClient> map = HTTPCLIENTS.get();
-        if ( map != null ) {
-            for ( HttpClient cl : map.values() ) {
+        Map<HttpClientKey, HttpClient> mapHttpClientPerHttpClientKey = HTTPCLIENTS_CACHE_PER_THREAD_AND_HTTPCLIENTKEY.get();
+        if ( mapHttpClientPerHttpClientKey != null ) {
+            for ( HttpClient cl : mapHttpClientPerHttpClientKey.values() ) {
                 ((AbstractHttpClient) cl).clearRequestInterceptors(); 
                 ((AbstractHttpClient) cl).clearResponseInterceptors(); 
                 cl.getConnectionManager().shutdown();
             }
-            map.clear();
+            mapHttpClientPerHttpClientKey.clear();
         }
     }
 
