@@ -22,6 +22,7 @@ import java.io.InputStream;
 import java.io.PrintStream;
 import java.io.UnsupportedEncodingException;
 import java.net.MalformedURLException;
+import java.net.URISyntaxException;
 import java.net.URL;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
@@ -53,6 +54,7 @@ import org.apache.jmeter.protocol.http.control.AuthManager;
 import org.apache.jmeter.protocol.http.control.CacheManager;
 import org.apache.jmeter.protocol.http.control.Cookie;
 import org.apache.jmeter.protocol.http.control.CookieManager;
+import org.apache.jmeter.protocol.http.control.DNSCacheManager;
 import org.apache.jmeter.protocol.http.control.HeaderManager;
 import org.apache.jmeter.protocol.http.parser.HTMLParseException;
 import org.apache.jmeter.protocol.http.parser.HTMLParser;
@@ -104,6 +106,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                     "org.apache.jmeter.protocol.http.config.gui.HttpDefaultsGui",
                     "org.apache.jmeter.config.gui.SimpleConfigGui",
                     "org.apache.jmeter.protocol.http.gui.HeaderPanel",
+                    "org.apache.jmeter.protocol.http.control.DNSCacheManager",
+                    "org.apache.jmeter.protocol.http.gui.DNSCachePanel",
                     "org.apache.jmeter.protocol.http.gui.AuthPanel",
                     "org.apache.jmeter.protocol.http.gui.CacheManagerGui",
                     "org.apache.jmeter.protocol.http.gui.CookiePanel"}));
@@ -119,6 +123,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     public static final String HEADER_MANAGER = "HTTPSampler.header_manager"; // $NON-NLS-1$
 
+    public static final String DNS_CACHE_MANAGER = "HTTPSampler.dns_cache_manager"; // $NON-NLS-1$
+
     public static final String DOMAIN = "HTTPSampler.domain"; // $NON-NLS-1$
 
     public static final String PORT = "HTTPSampler.port"; // $NON-NLS-1$
@@ -174,6 +180,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
     public static final String CONCURRENT_POOL = "HTTPSampler.concurrentPool"; // $NON-NLS-1$
 
     private static final String CONCURRENT_POOL_DEFAULT = "4"; // default for concurrent pool (do not change)
+    
+    private static final String USER_AGENT = "User-Agent"; // $NON-NLS-1$
 
     //- JMX names
 
@@ -224,6 +232,15 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         HTTPConstants.TRACE,
         HTTPConstants.DELETE,
         HTTPConstants.PATCH,
+        HTTPConstants.PROPFIND,
+        HTTPConstants.PROPPATCH,
+        HTTPConstants.MKCOL,
+        HTTPConstants.COPY,
+        HTTPConstants.MOVE,
+        HTTPConstants.LOCK,
+        HTTPConstants.UNLOCK,
+        HTTPConstants.REPORT,
+        HTTPConstants.MKCALENDAR
         };
 
     private static final List<String> METHODLIST = Collections.unmodifiableList(Arrays.asList(METHODS));
@@ -541,6 +558,9 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     /**
      * Add an argument which has already been encoded
+     *
+     * @param name name of the argument
+     * @param value value of the argument
      */
     public void addEncodedArgument(String name, String value) {
         this.addEncodedArgument(name, value, ARG_VAL_SEP);
@@ -621,6 +641,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
             setHeaderManager((HeaderManager) el);
         } else if (el instanceof AuthManager) {
             setAuthManager((AuthManager) el);
+        } else if (el instanceof DNSCacheManager) {
+            setDNSResolver((DNSCacheManager) el);
         } else {
             super.addTestElement(el);
         }
@@ -834,6 +856,18 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         return (CacheManager) getProperty(CACHE_MANAGER).getObjectValue();
     }
 
+    public DNSCacheManager getDNSResolver() {
+        return (DNSCacheManager) getProperty(DNS_CACHE_MANAGER).getObjectValue();
+    }
+
+    public void setDNSResolver(DNSCacheManager cacheManager) {
+        DNSCacheManager mgr = getDNSResolver();
+        if (mgr != null) {
+            log.warn("Existing DNSCacheManager " + mgr.getName() + " superseded by " + cacheManager.getName());
+        }
+        setProperty(new TestElementProperty(DNS_CACHE_MANAGER, cacheManager));
+    }
+
     public boolean isImageParser() {
         return getPropertyAsBoolean(IMAGE_PARSER, false);
     }
@@ -866,7 +900,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * @return the modified sampling result containing details of the Exception.
      */
     protected HTTPSampleResult errorResult(Throwable e, HTTPSampleResult res) {
-        res.setSampleLabel("Error: " + res.getSampleLabel());
+        res.setSampleLabel(res.getSampleLabel());
         res.setDataType(SampleResult.TEXT);
         ByteArrayOutputStream text = new ByteArrayOutputStream(200);
         e.printStackTrace(new PrintStream(text));
@@ -894,7 +928,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * </p>
      *
      * @return The URL to be requested by this sampler.
-     * @throws MalformedURLException
+     * @throws MalformedURLException if url is malformed
      */
     public URL getUrl() throws MalformedURLException {
         StringBuilder pathAndQuery = new StringBuilder(100);
@@ -1024,7 +1058,11 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      */
     public void parseArguments(String queryString, String contentEncoding) {
         String[] args = JOrphanUtils.split(queryString, QRY_SEP);
+        final boolean isDebug = log.isDebugEnabled();
         for (int i = 0; i < args.length; i++) {
+            if (isDebug) {
+                log.debug("Arg: " + args[i]);
+            }
             // need to handle four cases:
             // - string contains name=value
             // - string contains name=
@@ -1047,6 +1085,9 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                 value="";
             }
             if (name.length() > 0) {
+                if (isDebug) {
+                    log.debug("Name: " + name+ " Value: " + value+ " Metadata: " + metaData);
+                }
                 // If we know the encoding, we can decode the argument value,
                 // to make it easier to read for the user
                 if(!StringUtils.isEmpty(contentEncoding)) {
@@ -1103,7 +1144,9 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         SampleResult res = null;
         try {
             res = sample(getUrl(), getMethod(), false, 0);
-            res.setSampleLabel(getName());
+            if(res != null) {
+                res.setSampleLabel(getName());
+            }
             return res;
         } catch (Exception e) {
             return errorResult(e, new HTTPSampleResult());
@@ -1127,7 +1170,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * @param depth
      *            Depth of this target in the frame structure. Used only to
      *            prevent infinite recursion.
-     * @return results of the sampling
+     * @return results of the sampling, can be null if u is in CacheManager
      */
     protected abstract HTTPSampleResult sample(URL u,
             String method, boolean areFollowingRedirect, int depth);
@@ -1157,7 +1200,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                         HTMLParser.getParser(parserName)
                         :
                         HTMLParser.getParser(); // we don't; use the default parser
-                    urls = parser.getEmbeddedResourceURLs(responseData, res.getURL(), res.getDataEncodingWithDefault());
+                    String userAgent = getUserAgent(res);
+                    urls = parser.getEmbeddedResourceURLs(userAgent, responseData, res.getURL(), res.getDataEncodingWithDefault());
                 }
             }
         } catch (HTMLParseException e) {
@@ -1169,9 +1213,6 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         // Iterate through the URLs and download each image:
         if (urls != null && urls.hasNext()) {
             if (container == null) {
-                // TODO needed here because currently done on sample completion in JMeterThread,
-                // but that only catches top-level samples.
-                res.setThreadName(Thread.currentThread().getName());
                 container = new HTTPSampleResult(res);
                 container.addRawSubResult(res);
             }
@@ -1223,7 +1264,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                             // default: serial download embedded resources
                             HTTPSampleResult binRes = sample(url, HTTPConstants.GET, false, frameDepth + 1);
                             res.addSubResult(binRes);
-                            setParentSampleSuccess(res, res.isSuccessful() && binRes.isSuccessful());
+                            setParentSampleSuccess(res, res.isSuccessful() && (binRes != null ? binRes.isSuccessful() : true));
                         }
 
                     }
@@ -1287,7 +1328,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                                 }
                             }
                             res.addSubResult(binRes.getResult());
-                            setParentSampleSuccess(res, res.isSuccessful() && binRes.getResult().isSuccessful());
+                            setParentSampleSuccess(res, res.isSuccessful() && (binRes.getResult() != null ? binRes.getResult().isSuccessful():true));
                         } catch (TimeoutException e) {
                             errorResult(e, res);
                         }
@@ -1307,6 +1348,33 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         return res;
     }
     
+    /**
+     * Extract User-Agent header value
+     * @param sampleResult HTTPSampleResult
+     * @return User Agent part
+     */
+    private String getUserAgent(HTTPSampleResult sampleResult) {
+        String res = sampleResult.getRequestHeaders();
+        int index = res.indexOf(USER_AGENT);
+        if(index >=0) {
+            // see HTTPHC3Impl#getConnectionHeaders
+            // see HTTPHC4Impl#getConnectionHeaders
+            // see HTTPJavaImpl#getConnectionHeaders    
+            //': ' is used by JMeter to fill-in requestHeaders, see getConnectionHeaders
+            final String userAgentPrefix = USER_AGENT+": ";
+            String userAgentHdr = res.substring(
+                    index+userAgentPrefix.length(), 
+                    res.indexOf('\n',// '\n' is used by JMeter to fill-in requestHeaders, see getConnectionHeaders
+                            index+userAgentPrefix.length()+1));
+            return userAgentHdr.trim();
+        } else {
+            if(log.isInfoEnabled()) {
+                log.info("No user agent extracted from requestHeaders:"+res);
+            }
+            return null;
+        }
+    }
+
     /**
      * Set parent successful attribute based on IGNORE_FAILED_EMBEDDED_RESOURCES parameter
      * @param res {@link HTTPSampleResult}
@@ -1395,6 +1463,9 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         for (redirect = 0; redirect < MAX_REDIRECTS; redirect++) {
             boolean invalidRedirectUrl = false;
             String location = lastRes.getRedirectLocation(); 
+            if (log.isDebugEnabled()) {
+                log.debug("Initial location: " + location);
+            }
             if (REMOVESLASHDOTDOT) {
                 location = ConversionUtils.removeSlashDotDot(location);
             }
@@ -1402,17 +1473,35 @@ public abstract class HTTPSamplerBase extends AbstractSampler
             // replacing them automatically with %20. We want to emulate
             // this behaviour.
             location = encodeSpaces(location);
+            if (log.isDebugEnabled()) {
+                log.debug("Location after /. and space transforms: " + location);
+            }
             // Change all but HEAD into GET (Bug 55450)
             String method = lastRes.getHTTPMethod();
             if (!HTTPConstants.HEAD.equalsIgnoreCase(method)) {
                 method = HTTPConstants.GET;
             }
             try {
-                lastRes = sample(ConversionUtils.makeRelativeURL(lastRes.getURL(), location), method, true, frameDepth);
+                URL url = ConversionUtils.makeRelativeURL(lastRes.getURL(), location);
+                url = ConversionUtils.sanitizeUrl(url).toURL();
+                if (log.isDebugEnabled()) {
+                    log.debug("Location as URL: " + url.toString());
+                }
+                HTTPSampleResult tempRes = sample(url, method, true, frameDepth);
+                if(tempRes != null) {
+                    lastRes = tempRes;
+                } else {
+                    // Last url was in cache so tempRes is null
+                    break;
+                }
             } catch (MalformedURLException e) {
                 errorResult(e, lastRes);
                 // The redirect URL we got was not a valid URL
                 invalidRedirectUrl = true;
+            } catch (URISyntaxException e) {
+                errorResult(e, lastRes);
+                // The redirect URL we got was not a valid URL
+                invalidRedirectUrl = true;
             }
             if (lastRes.getSubResults() != null && lastRes.getSubResults().length > 0) {
                 SampleResult[] subs = lastRes.getSubResults();
@@ -1467,9 +1556,9 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * an HTTPSamplerResult container parameter instead of a
      * boolean:areFollowingRedirect.
      *
-     * @param areFollowingRedirect
-     * @param frameDepth
-     * @param res
+     * @param areFollowingRedirect flag whether we are getting a redirect target
+     * @param frameDepth Depth of this target in the frame structure. Used only to prevent infinite recursion.
+     * @param res sample result to process
      * @return the sample result
      */
     protected HTTPSampleResult resultProcessing(boolean areFollowingRedirect, int frameDepth, HTTPSampleResult res) {
@@ -1509,6 +1598,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * Determine if the HTTP status code is successful or not
      * i.e. in range 200 to 399 inclusive
      *
+     * @param code status code to check
      * @return whether in range 200-399 or not
      */
     protected boolean isSuccessCode(int code){
@@ -1616,16 +1706,16 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     /**
      * Read response from the input stream, converting to MD5 digest if the useMD5 property is set.
-     *
+     * <p>
      * For the MD5 case, the result byte count is set to the size of the original response.
-     * 
+     * <p>
      * Closes the inputStream 
      * 
-     * @param sampleResult
-     * @param in input stream
+     * @param sampleResult sample to store information about the response into
+     * @param in input stream from which to read the response
      * @param length expected input length or zero
      * @return the response or the MD5 of the response
-     * @throws IOException
+     * @throws IOException if reading the result fails
      */
     public byte[] readResponse(SampleResult sampleResult, InputStream in, int length) throws IOException {
         try {
@@ -1683,18 +1773,20 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     /**
      * JMeter 2.3.1 and earlier only had fields for one file on the GUI:
-     * - FILE_NAME
-     * - FILE_FIELD
-     * - MIMETYPE
+     * <ul>
+     *   <li>FILE_NAME</li>
+     *   <li>FILE_FIELD</li>
+     *   <li>MIMETYPE</li>
+     * </ul>
      * These were stored in their own individual properties.
-     *
+     * <p>
      * Version 2.3.3 introduced a list of files, each with their own path, name and mimetype.
-     *
+     * <p>
      * In order to maintain backwards compatibility of test plans, the 3 original properties
      * were retained; additional file entries are stored in an HTTPFileArgs class.
      * The HTTPFileArgs class was only present if there is more than 1 file; this means that
      * such test plans are backward compatible.
-     *
+     * <p>
      * Versions after 2.3.4 dispense with the original set of 3 properties.
      * Test plans that use them are converted to use a single HTTPFileArgs list.
      *
@@ -1732,6 +1824,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     /**
      * set IP source to use - does not apply to Java HTTP implementation currently
+     *
+     * @param value IP source to use
      */
     public void setIpSource(String value) {
         setProperty(IP_SOURCE, value, "");
@@ -1739,6 +1833,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     /**
      * get IP source to use - does not apply to Java HTTP implementation currently
+     *
+     * @return IP source to use
      */
     public String getIpSource() {
         return getPropertyAsString(IP_SOURCE,"");
@@ -1746,6 +1842,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
  
     /**
      * set IP/address source type to use
+     *
+     * @param value type of the IP/address source
      */
     public void setIpSourceType(int value) {
         setProperty(IP_SOURCE_TYPE, value, SOURCE_TYPE_DEFAULT);
@@ -1872,8 +1970,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         private final HTTPSampleResult result;
         private final CollectionProperty cookies;
         /**
-         * @param result
-         * @param cookies
+         * @param result {@link HTTPSampleResult} to hold
+         * @param cookies cookies to hold
          */
         public AsynSamplerResultHolder(HTTPSampleResult result, CollectionProperty cookies) {
             super();
@@ -1902,4 +2000,4 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         String guiClass = configElement.getProperty(TestElement.GUI_CLASS).getStringValue();
         return APPLIABLE_CONFIG_CLASSES.contains(guiClass);
     }
-}
\ No newline at end of file
+}
