@@ -19,7 +19,6 @@
 package org.apache.jmeter.protocol.jms.sampler;
 
 import java.util.Date;
-import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Map;
 
@@ -71,6 +70,10 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
 
     private static final String TIMEOUT = "JMSSampler.timeout"; // $NON-NLS-1$
 
+    private static final String JMS_PRIORITY = "JMSSampler.priority"; // $NON-NLS-1$
+
+    private static final String JMS_EXPIRATION = "JMSSampler.expiration"; // $NON-NLS-1$
+
     private static final String JMS_SELECTOR = "JMSSampler.jmsSelector"; // $NON-NLS-1$
 
     private static final String JMS_SELECTOR_DEFAULT = ""; // $NON-NLS-1$
@@ -124,6 +127,8 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
 
     private transient Throwable thrown = null;
 
+    private transient Context context = null;
+
     /**
      * {@inheritDoc}
      */
@@ -139,7 +144,10 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
         try {
             TextMessage msg = createMessage();
             if (isOneway()) {
-                producer.send(msg);
+                int deliveryMode = isNonPersistent() ? 
+                        DeliveryMode.NON_PERSISTENT:DeliveryMode.PERSISTENT;
+                producer.send(msg, deliveryMode, Integer.parseInt(getPriority()), 
+                        Long.parseLong(getExpiration()));
                 res.setRequestHeaders(Utils.messageProperties(msg));
                 res.setResponseOK();
                 res.setResponseData("Oneway request has no response data", null);
@@ -147,8 +155,10 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
                 if (!useTemporyQueue()) {
                     msg.setJMSReplyTo(receiveQueue);
                 }
-
-                Message replyMsg = executor.sendAndReceive(msg);
+                Message replyMsg = executor.sendAndReceive(msg,
+                        isNonPersistent() ? DeliveryMode.NON_PERSISTENT : DeliveryMode.PERSISTENT, 
+                        Integer.parseInt(getPriority()), 
+                        Long.parseLong(getExpiration()));
                 res.setRequestHeaders(Utils.messageProperties(msg));
                 if (replyMsg == null) {
                     res.setResponseMessage("No reply message received");
@@ -313,7 +323,6 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     public void threadStarted() {
         logThreadStart();
 
-        Context context = null;
         thrown = null;
         try {
             context = getInitialContext();
@@ -329,15 +338,16 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
 
             if (!useTemporyQueue()) {
                 receiveQueue = (Queue) context.lookup(getReceiveQueue());
-                receiverThread = Receiver.createReceiver(factory, receiveQueue, getPrincipal(context), getCredentials(context)
+                receiverThread = Receiver.createReceiver(factory, receiveQueue, Utils.getFromEnvironment(context, Context.SECURITY_PRINCIPAL), 
+                        Utils.getFromEnvironment(context, Context.SECURITY_CREDENTIALS)
                         , isUseResMsgIdAsCorrelId(), getJMSSelector());
             }
 
             String principal = null;
             String credentials = null;
             if (USE_SECURITY_PROPERTIES){
-                principal = getPrincipal(context);
-                credentials = getCredentials(context);
+                principal = Utils.getFromEnvironment(context, Context.SECURITY_PRINCIPAL);
+                credentials = Utils.getFromEnvironment(context, Context.SECURITY_CREDENTIALS);
             }
             if (principal != null && credentials != null) {
                 connection = factory.createQueueConnection(principal, credentials);
@@ -351,20 +361,19 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
                 LOGGER.debug("Session created");
             }
 
-            if (getPropertyAsBoolean(IS_ONE_WAY)) {
+            if (isOneway()) {
                 producer = session.createSender(sendQueue);
                 if (isNonPersistent()) {
                     producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
                 }
+                producer.setPriority(Integer.parseInt(getPriority()));
+                producer.setTimeToLive(Long.parseLong(getExpiration()));
             } else {
 
                 if (useTemporyQueue()) {
                     executor = new TemporaryQueueExecutor(session, sendQueue);
                 } else {
                     producer = session.createSender(sendQueue);
-                    if (isNonPersistent()) {
-                        producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
-                    }
                     executor = new FixedQueueExecutor(producer, getTimeoutAsInt(), isUseReqMsgIdAsCorrelId());
                 }
             }
@@ -383,14 +392,6 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
         } catch (NoClassDefFoundError e) {
             thrown = e;
             LOGGER.error(e.getLocalizedMessage(), e);
-        } finally {
-            if (context != null) {
-                try {
-                    context.close();
-                } catch (NamingException ignored) {
-                    // ignore
-                }
-            }
         }
     }
 
@@ -429,13 +430,19 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     }
 
     private void printEnvironment(Context context) throws NamingException {
-        Hashtable<?,?> env = context.getEnvironment();
-        LOGGER.debug("Initial Context Properties");
-        @SuppressWarnings("unchecked")
-        Enumeration<String> keys = (Enumeration<String>) env.keys();
-        while (keys.hasMoreElements()) {
-            String key = keys.nextElement();
-            LOGGER.debug(key + "=" + env.get(key));
+        try {
+            Hashtable<?,?> env = context.getEnvironment();
+            if(env != null) {
+                LOGGER.debug("Initial Context Properties");
+                for (Map.Entry<?, ?> entry : env.entrySet()) {
+                    LOGGER.debug(entry.getKey() + "=" + entry.getValue());
+                }
+            } else {
+                LOGGER.warn("context.getEnvironment() returned null (should not happen according to javadoc but non compliant implementation can return this)");
+            }
+        } catch (javax.naming.OperationNotSupportedException ex) {
+            // Some JNDI implementation can return this
+            LOGGER.warn("context.getEnvironment() not supported by implementation ");
         }
     }
 
@@ -460,7 +467,25 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     public String getTimeout() {
         return getPropertyAsString(TIMEOUT, DEFAULT_TIMEOUT_STRING);
     }
+    
+    public String getExpiration() {
+        String expiration = getPropertyAsString(JMS_EXPIRATION);
+        if (expiration.length() == 0) {
+            return Utils.DEFAULT_NO_EXPIRY;
+        } else {
+            return expiration;
+        }
+    }
 
+    public String getPriority() {
+        String priority = getPropertyAsString(JMS_PRIORITY);
+        if (priority.length() == 0) {
+            return Utils.DEFAULT_PRIORITY_4;
+        } else {
+            return priority;
+        }
+    }
+    
     /**
      * {@inheritDoc}
      */
@@ -468,6 +493,13 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     public void threadFinished() {
         LOGGER.debug("Thread ended " + new Date());
 
+        if (context != null) {
+            try {
+                context.close();
+            } catch (NamingException ignored) {
+                // ignore
+            }
+        }
         Utils.close(session, LOGGER);
         Utils.close(connection, LOGGER);
         if (receiverThread != null) {
@@ -491,6 +523,14 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     public void setTimeout(String s) {
         setProperty(JMSSampler.TIMEOUT, s);
     }
+    
+    public void setPriority(String s) {
+        setProperty(JMSSampler.JMS_PRIORITY, s, Utils.DEFAULT_PRIORITY_4);
+    }
+    
+    public void setExpiration(String s) {
+        setProperty(JMSSampler.JMS_EXPIRATION, s, Utils.DEFAULT_NO_EXPIRY);
+    }
 
     /**
      * @return String JMS Selector
@@ -507,7 +547,7 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     }
     
     /**
-     * @param string
+     * @param string name of the initial context factory to use
      */
     public void setInitialContextFactory(String string) {
         setProperty(JNDI_INITIAL_CONTEXT_FACTORY, string);
@@ -515,34 +555,10 @@ public class JMSSampler extends AbstractSampler implements ThreadListener {
     }
 
     /**
-     * @param string
+     * @param string url of the provider
      */
     public void setContextProvider(String string) {
         setProperty(JNDI_CONTEXT_PROVIDER_URL, string);
 
     }
-
-    /**
-     * get the principal from the context property java.naming.security.principal
-     *
-     * @param context
-     * @return the principal
-     * @throws NamingException
-     */
-    private String getPrincipal(Context context) throws NamingException{
-            Hashtable<?,?> env = context.getEnvironment();
-            return (String) env.get("java.naming.security.principal"); // $NON-NLS-1$
-    }
-
-    /**
-     * get the credentials from the context property java.naming.security.credentials
-     *
-     * @param context
-     * @return the credentials
-     * @throws NamingException
-     */
-    private String getCredentials(Context context) throws NamingException{
-            Hashtable<?,?> env = context.getEnvironment();
-            return (String) env.get("java.naming.security.credentials"); // $NON-NLS-1$
-    }
 }
