@@ -27,6 +27,7 @@ import java.lang.reflect.Method;
 import java.util.HashMap;
 import java.util.Map;
 
+import javax.jms.DeliveryMode;
 import javax.jms.JMSException;
 import javax.jms.Message;
 import javax.naming.NamingException;
@@ -74,6 +75,10 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     
     private static final String JMS_PROPERTIES = "jms.jmsProperties"; // $NON-NLS-1$
 
+    private static final String JMS_PRIORITY = "jms.priority"; // $NON-NLS-1$
+
+    private static final String JMS_EXPIRATION = "jms.expiration"; // $NON-NLS-1$
+
     //--
 
     // Does not need to be synch. because it is only accessed from the sampler thread
@@ -132,7 +137,7 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     private void initClient() throws JMSException, NamingException {
         publisher = new Publisher(getUseJNDIPropertiesAsBoolean(), getJNDIInitialContextFactory(), 
                 getProviderUrl(), getConnectionFactory(), getDestination(), isUseAuth(), getUsername(),
-                getPassword(), isDestinationStatic(), getUseNonPersistentDelivery());
+                getPassword(), isDestinationStatic());
         ClientPool.addClient(publisher);
         log.debug("PublisherSampler.initClient called");
     }
@@ -168,23 +173,27 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
         
         try {
             Map<String, Object> msgProperties = getJMSProperties().getJmsPropertysAsMap();
+            int deliveryMode = getUseNonPersistentDelivery() ? DeliveryMode.NON_PERSISTENT : DeliveryMode.PERSISTENT; 
+            int priority = Integer.parseInt(getPriority());
+            long expiration = Long.parseLong(getExpiration());
+            
             for (int idx = 0; idx < loop; idx++) {
                 if (JMSPublisherGui.TEXT_MSG_RSC.equals(type)){
                     String tmsg = getMessageContent();
-                    Message msg = publisher.publish(tmsg, getDestination(), msgProperties);
+                    Message msg = publisher.publish(tmsg, getDestination(), msgProperties, deliveryMode, priority, expiration);
                     buffer.append(tmsg);
                     Utils.messageProperties(propBuffer, msg);
                 } else if (JMSPublisherGui.MAP_MSG_RSC.equals(type)){
                     Map<String, Object> m = getMapContent();
-                    Message msg = publisher.publish(m, getDestination(), msgProperties);
+                    Message msg = publisher.publish(m, getDestination(), msgProperties, deliveryMode, priority, expiration);
                     Utils.messageProperties(propBuffer, msg);
                 } else if (JMSPublisherGui.OBJECT_MSG_RSC.equals(type)){
                     Serializable omsg = getObjectContent();
-                    Message msg = publisher.publish(omsg, getDestination(), msgProperties);
+                    Message msg = publisher.publish(omsg, getDestination(), msgProperties, deliveryMode, priority, expiration);
                     Utils.messageProperties(propBuffer, msg);
                 } else if (JMSPublisherGui.BYTES_MSG_RSC.equals(type)){
                     byte[] bmsg = getBytesContent();
-                    Message msg = publisher.publish(bmsg, getDestination(), msgProperties);
+                    Message msg = publisher.publish(bmsg, getDestination(), msgProperties, deliveryMode, priority, expiration);
                     Utils.messageProperties(propBuffer, msg);
                 } else {
                     throw new JMSException(type+ " is not recognised");                    
@@ -260,7 +269,7 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
      * The implementation uses TextFile to load the contents of the file and
      * returns a string.
      *
-     * @param path
+     * @param path path to the file to read in
      * @return the contents of the file
      */
     public String getFileContent(String path) {
@@ -395,6 +404,10 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
      * set the source of the message
      *
      * @param choice
+     *            source of the messages. One of
+     *            {@link JMSPublisherGui#USE_FILE_RSC},
+     *            {@link JMSPublisherGui#USE_RANDOM_RSC} or
+     *            JMSPublisherGui#USE_TEXT_RSC
      */
     public void setConfigChoice(String choice) {
         setProperty(CONFIG_CHOICE, choice);
@@ -407,6 +420,8 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     /**
      * return the source of the message
      * Converts from old JMX files which used the local language string
+     *
+     * @return source of the messages
      */
     public String getConfigChoice() {
         // Allow for the old JMX file which used the local language string
@@ -425,14 +440,14 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     /**
      * set the type of the message
      *
-     * @param choice
+     * @param choice type of the message (Text, Object, Map)
      */
     public void setMessageChoice(String choice) {
         setProperty(MESSAGE_CHOICE, choice);
     }
 
     /**
-     * return the type of the message (Text, Object, Map)
+     * @return the type of the message (Text, Object, Map)
      *
      */
     public String getMessageChoice() {
@@ -442,14 +457,14 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     /**
      * set the input file for the publisher
      *
-     * @param file
+     * @param file input file for the publisher
      */
     public void setInputFile(String file) {
         setProperty(INPUT_FILE, file);
     }
 
     /**
-     * return the path of the input file
+     * @return the path of the input file
      *
      */
     public String getInputFile() {
@@ -459,14 +474,14 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     /**
      * set the random path for the messages
      *
-     * @param path
+     * @param path random path for the messages
      */
     public void setRandomPath(String path) {
         setProperty(RANDOM_PATH, path);
     }
 
     /**
-     * return the random path for messages
+     * @return the random path for messages
      *
      */
     public String getRandomPath() {
@@ -476,20 +491,36 @@ public class PublisherSampler extends BaseJMSSampler implements TestStateListene
     /**
      * set the text for the message
      *
-     * @param message
+     * @param message text for the message
      */
     public void setTextMessage(String message) {
         setProperty(TEXT_MSG, message);
     }
 
     /**
-     * return the text for the message
+     * @return the text for the message
      *
      */
     public String getTextMessage() {
         return getPropertyAsString(TEXT_MSG);
     }
 
+    public String getExpiration() {
+        return getPropertyAsString(JMS_EXPIRATION, Utils.DEFAULT_NO_EXPIRY);
+    }
+
+    public String getPriority() {
+        return getPropertyAsString(JMS_PRIORITY, Utils.DEFAULT_PRIORITY_4);
+    }
+    
+    public void setPriority(String s) {
+        setProperty(JMS_PRIORITY, s, Utils.DEFAULT_PRIORITY_4);
+    }
+    
+    public void setExpiration(String s) {
+        setProperty(JMS_EXPIRATION, s, Utils.DEFAULT_NO_EXPIRY);
+    }
+    
     /**
      * @param value boolean use NON_PERSISTENT
      */
