@@ -79,7 +79,9 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
     private static final String DELETE = "delete"; // $NON-NLS-1$
     private static final String NUM_MESSAGES = "num_messages"; // $NON-NLS-1$
     private static final String NEW_LINE = "\n"; // $NON-NLS-1$
-    private static final String STORE_MIME_MESSAGE = "storeMimeMessage";
+    private static final String STORE_MIME_MESSAGE = "storeMimeMessage"; // $NON-NLS-1$
+    private static final String HEADER_ONLY = "headerOnly"; // $NON-NLS-1$
+    private static final boolean HEADER_ONLY_DEFAULT = false;
     //-
 
     private static final String RFC_822_DEFAULT_ENCODING = "iso-8859-1"; // RFC 822 uses ascii per default
@@ -89,6 +91,10 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
     // Use the actual class so the name must be correct.
     private static final String TRUST_ALL_SOCKET_FACTORY = TrustAllSSLSocketFactory.class.getName();
 
+    private static final String FALSE = "false";  // $NON-NLS-1$
+
+    private static final String TRUE = "true";  // $NON-NLS-1$
+
     public boolean isUseLocalTrustStore() {
         return getPropertyAsBoolean(SecuritySettingsPanel.USE_LOCAL_TRUSTSTORE);
     }
@@ -136,7 +142,8 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
     public SampleResult sample(Entry e) {
         SampleResult parent = new SampleResult();
         boolean isOK = false; // Did sample succeed?
-        boolean deleteMessages = getDeleteMessages();
+        final boolean deleteMessages = getDeleteMessages();
+        final String serverProtocol = getServerType();
 
         parent.setSampleLabel(getName());
 
@@ -152,20 +159,20 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
             Properties props = new Properties();
 
             if (isUseStartTLS()) {
-                props.setProperty("mail.pop3s.starttls.enable", "true");
+                props.setProperty(mailProp(serverProtocol, "starttls.enable"), TRUE);  // $NON-NLS-1$
                 if (isEnforceStartTLS()){
                     // Requires JavaMail 1.4.2+
-                    props.setProperty("mail.pop3s.starttls.require", "true");
+                    props.setProperty(mailProp(serverProtocol, "starttls.require"), TRUE);  // $NON-NLS-1$
                 }
             }
 
             if (isTrustAllCerts()) {
                 if (isUseSSL()) {
-                    props.setProperty("mail.pop3s.ssl.socketFactory.class", TRUST_ALL_SOCKET_FACTORY);
-                    props.setProperty("mail.pop3s.ssl.socketFactory.fallback", "false");
+                    props.setProperty(mailProp(serverProtocol, "ssl.socketFactory.class"), TRUST_ALL_SOCKET_FACTORY);  // $NON-NLS-1$
+                    props.setProperty(mailProp(serverProtocol, "ssl.socketFactory.fallback"), FALSE);  // $NON-NLS-1$
                 } else if (isUseStartTLS()) {
-                    props.setProperty("mail.pop3s.ssl.socketFactory.class", TRUST_ALL_SOCKET_FACTORY);
-                    props.setProperty("mail.pop3s.ssl.socketFactory.fallback", "false");
+                    props.setProperty(mailProp(serverProtocol, "ssl.socketFactory.class"), TRUST_ALL_SOCKET_FACTORY);  // $NON-NLS-1$
+                    props.setProperty(mailProp(serverProtocol, "ssl.socketFactory.fallback"), FALSE);  // $NON-NLS-1$
                 }
             } else if (isUseLocalTrustStore()){
                 File truststore = new File(getTrustStoreToUse());
@@ -181,12 +188,14 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
                 }
                 if (isUseSSL()) {
                     // Requires JavaMail 1.4.2+
-                    props.put("mail.pop3s.ssl.socketFactory", new LocalTrustStoreSSLSocketFactory(truststore));
-                    props.put("mail.pop3s.ssl.socketFactory.fallback", "false");
+                    props.put(mailProp(serverProtocol, "ssl.socketFactory"),   // $NON-NLS-1$ 
+                            new LocalTrustStoreSSLSocketFactory(truststore));
+                    props.put(mailProp(serverProtocol, "ssl.socketFactory.fallback"), FALSE);  // $NON-NLS-1$
                 } else if (isUseStartTLS()) {
                     // Requires JavaMail 1.4.2+
-                    props.put("mail.pop3s.ssl.socketFactory", new LocalTrustStoreSSLSocketFactory(truststore));
-                    props.put("mail.pop3s.ssl.socketFactory.fallback", "false");
+                    props.put(mailProp(serverProtocol, "ssl.socketFactory"),  // $NON-NLS-1$
+                            new LocalTrustStoreSSLSocketFactory(truststore));
+                    props.put(mailProp(serverProtocol, "ssl.socketFactory.fallback"), FALSE);  // $NON-NLS-1$
                 }
             }
 
@@ -194,7 +203,7 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
             Session session = Session.getInstance(props, null);
 
             // Get the store
-            Store store = session.getStore(getServerType());
+            Store store = session.getStore(serverProtocol);
             store.connect(getServer(), getPortAsInt(), getUserName(), getPassword());
 
             // Get folder
@@ -205,8 +214,14 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
                 folder.open(Folder.READ_ONLY);
             }
 
+            final int messageTotal = folder.getMessageCount();
+            int n = getNumMessages();
+            if (n == ALL_MESSAGES || n > messageTotal) {
+                n = messageTotal;
+            }
+
             // Get directory
-            Message messages[] = folder.getMessages();
+            Message messages[] = folder.getMessages(1,n);
             StringBuilder pdata = new StringBuilder();
             pdata.append(messages.length);
             pdata.append(" messages found\n");
@@ -214,19 +229,12 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
             parent.setDataType(SampleResult.TEXT);
             parent.setContentType("text/plain"); // $NON-NLS-1$
 
-            int n = getNumMessages();
-            if (n == ALL_MESSAGES || n > messages.length) {
-                n = messages.length;
-            }
-
-            parent.setSampleCount(n); // TODO is this sensible?
-
+            final boolean headerOnly = getHeaderOnly();
             busy = true;
-            for (int i = 0; busy && i < n; i++) {
+            for (Message message : messages) {
                 StringBuilder cdata = new StringBuilder();
                 SampleResult child = new SampleResult();
                 child.sampleStart();
-                Message message = messages[i];
 
                 cdata.append("Message "); // $NON-NLS-1$
                 cdata.append(message.getMessageNumber());
@@ -260,7 +268,9 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
                     }
                     child.setResponseHeaders(cdata.toString());
                     cdata.setLength(0);
-                    appendMessageData(child, message);
+                    if (!headerOnly) {
+                        appendMessageData(child, message);
+                    }
                 }
 
                 if (deleteMessages) {
@@ -384,7 +394,7 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
      * server. Either MailReaderSampler.TYPE_IMAP[S] or
      * MailReaderSampler.TYPE_POP3[S]. Default is MailReaderSampler.TYPE_POP3.
      *
-     * @param serverType
+     * @param serverType protocol to use
      */
     public void setServerType(String serverType) {
         setProperty(SERVER_TYPE, serverType);
@@ -444,7 +454,7 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
     }
 
     /**
-     * @param password
+     * @param password the password to use
      */
     public void setPassword(String password) {
         setProperty(PASSWORD, password);
@@ -580,4 +590,26 @@ public class MailReaderSampler extends AbstractSampler implements Interruptible
         String guiClass = configElement.getProperty(TestElement.GUI_CLASS).getStringValue();
         return APPLIABLE_CONFIG_CLASSES.contains(guiClass);
     }
+
+    public boolean getHeaderOnly() {
+        return getPropertyAsBoolean(HEADER_ONLY, HEADER_ONLY_DEFAULT);
+    }
+
+    public void setHeaderOnly(boolean selected) {
+        setProperty(HEADER_ONLY, selected, HEADER_ONLY_DEFAULT);
+    }
+
+    /**
+     * Build a property name of the form "mail.pop3s.starttls.require"
+     *
+     * @param protocol the protocol, i.e. "pop3s" in the example
+     * @param propname the property name suffix, i.e. "starttls.require" in the example
+     * @return the constructed name
+     */
+    private String mailProp(String protocol, String propname) {
+        StringBuilder sb = new StringBuilder();
+        sb.append("mail.").append(protocol).append(".");
+        sb.append(propname);
+        return sb.toString();
+    }
 }
