@@ -90,6 +90,9 @@ public class SendMailCommand {
 
     private String mailBody;
 
+    private String timeOut; // Socket read timeout value in milliseconds. This timeout is implemented by java.net.Socket.
+    private String connectionTimeOut; // Socket connection timeout value in milliseconds. This timeout is implemented by java.net.Socket.
+
     // case we are measuring real time of spedition
     private boolean synchronousMode;
 
@@ -114,7 +117,10 @@ public class SendMailCommand {
      *
      * @return Message-object to be sent to execute()-method
      * @throws MessagingException
+     *             when problems constructing or sending the mail occur
      * @throws IOException
+     *             when the mail content can not be read or truststore problems
+     *             are detected
      */
     public Message prepareMessage() throws MessagingException, IOException {
 
@@ -126,6 +132,10 @@ public class SendMailCommand {
         props.setProperty("mail." + protocol + ".host", smtpServer);
         props.setProperty("mail." + protocol + ".port", getPort());
         props.setProperty("mail." + protocol + ".auth", Boolean.toString(useAuthentication));
+        
+        // set timeout
+        props.setProperty("mail." + protocol + ".timeout", getTimeout());
+        props.setProperty("mail." + protocol + ".connectiontimeout", getConnectionTimeout());
 
         if (enableDebug) {
             props.setProperty("mail.debug","true");
@@ -251,13 +261,18 @@ public class SendMailCommand {
     }
 
     /**
-     * Sends message to mailserver, waiting for delivery if using synchronous mode.
+     * Sends message to mailserver, waiting for delivery if using synchronous
+     * mode.
      *
      * @param message
-     *            Message prior prepared by prepareMessage()
+     *            Message previously prepared by prepareMessage()
      * @throws MessagingException
+     *             when problems sending the mail arise
      * @throws IOException
+     *             TODO can not see how
      * @throws InterruptedException
+     *             when interrupted while waiting for delivery in synchronous
+     *             modus
      */
     public void execute(Message message) throws MessagingException, IOException, InterruptedException {
 
@@ -291,9 +306,16 @@ public class SendMailCommand {
     /**
      * Processes prepareMessage() and execute()
      *
-     * @throws Exception
+     * @throws InterruptedException
+     *             when interrupted while waiting for delivery in synchronous
+     *             modus
+     * @throws IOException
+     *             when the mail content can not be read or truststore problems
+     *             are detected
+     * @throws MessagingException
+     *             when problems constructing or sending the mail occur
      */
-    public void execute() throws Exception {
+    public void execute() throws MessagingException, IOException, InterruptedException {
         execute(prepareMessage());
     }
 
@@ -402,14 +424,14 @@ public class SendMailCommand {
      * SmtpSampler-object
      *
      * @param receiverTo
-     *            List of receivers <InternetAddress>
+     *            List of receivers
      */
     public void setReceiverTo(List<InternetAddress> receiverTo) {
         this.receiverTo = receiverTo;
     }
 
     /**
-     * Returns receivers of current message <InternetAddress> ("cc") - standard
+     * Returns receivers of current message as {@link InternetAddress} ("cc") - standard
      * getter
      *
      * @return List of receivers
@@ -423,14 +445,14 @@ public class SendMailCommand {
      * SmtpSampler-object
      *
      * @param receiverCC
-     *            List of receivers <InternetAddress>
+     *            List of receivers
      */
     public void setReceiverCC(List<InternetAddress> receiverCC) {
         this.receiverCC = receiverCC;
     }
 
     /**
-     * Returns receivers of current message <InternetAddress> ("bcc") - standard
+     * Returns receivers of current message as {@link InternetAddress} ("bcc") - standard
      * getter
      *
      * @return List of receivers
@@ -444,7 +466,7 @@ public class SendMailCommand {
      * SmtpSampler-object
      *
      * @param receiverBCC
-     *            List of receivers <InternetAddress>
+     *            List of receivers
      */
     public void setReceiverBCC(List<InternetAddress> receiverBCC) {
         this.receiverBCC = receiverBCC;
@@ -708,6 +730,48 @@ public class SendMailCommand {
         return "25";
     }
 
+    /**
+     * @param timeOut the timeOut to set
+     */
+    public void setTimeOut(String timeOut) {
+        this.timeOut = timeOut;
+    }
+
+    /**
+     * Returns timeout for the SMTP-connection - returns the
+     * default timeout if no value has been supplied.
+     *
+     * @return Timeout to be set for SMTP-connection
+     */
+    public String getTimeout() {
+        String timeout = timeOut.trim();
+        if (timeout.length() > 0) { // OK, it has been supplied
+            return timeout;
+        }
+        return "0"; // Default is infinite timeout (value 0).
+    }
+
+    /**
+     * @param connectionTimeOut the connectionTimeOut to set
+     */
+    public void setConnectionTimeOut(String connectionTimeOut) {
+        this.connectionTimeOut = connectionTimeOut;
+    }
+
+    /**
+     * Returns connection timeout for the SMTP-connection - returns the
+     * default connection timeout if no value has been supplied.
+     *
+     * @return Connection timeout to be set for SMTP-connection
+     */
+    public String getConnectionTimeout() {
+        String connectionTimeout = connectionTimeOut.trim();
+        if (connectionTimeout.length() > 0) { // OK, it has been supplied
+            return connectionTimeout;
+        }
+        return "0"; // Default is infinite timeout (value 0).
+    }
+
     /**
      * Assigns the object to use a local truststore for SSL / StartTLS - to be
      * called by SmtpSampler-object
@@ -747,7 +811,7 @@ public class SendMailCommand {
     /**
      * Set the mail body.
      *
-     * @param body
+     * @param body the body of the mail
      */
     public void setMailBody(String body){
         mailBody = body;
@@ -774,4 +838,4 @@ public class SendMailCommand {
     public void setReplyTo(List<InternetAddress> replyTo) {
         this.replyTo = replyTo;
     }
-}
\ No newline at end of file
+}
